@@ -29,14 +29,12 @@ def main():
     with open(input_fname) as in_file, open(sys.argv[2], 'w') as out_file:
         out_file.write('{"sections" : [')
 
-        line = in_file.readline()
-        while line:
+        for _ in iter(in_file.readline, ''):
             current_line += 1
             if 0 == current_line % lines_per_chunk:
                 chunk_end = in_file.tell()
                 out_file.write(f'{{"start":"{chunk_begin}","end":"{chunk_end}","sequences":"{sequences}"}},')
                 chunk_begin = chunk_end
-            line = in_file.readline()
 
         chunk_end = in_file.tell()
         out_file.write(f'{{"start":"{chunk_begin}","end":"{chunk_end}","sequences":"{current_line % lines_per_chunk / 4}"}}')
