@@ -19,7 +19,7 @@ assert sys.version_info[:2] >= (2, 4)
 
 
 def stop_err(msg):
-    sys.exit("%s" % msg)
+    sys.exit(f"{msg}")
 
 
 def __main__():
@@ -57,9 +57,9 @@ def __main__():
                 if quality_title and read_title != quality_title:
                     stop_err('Invalid fastqsolexa format at line %d: sequence title "%s" differes from score title "%s".' % (i + 1, read_title, quality_title))
                 if not quality_title:
-                    outfile_score.write('>%s\n' % read_title)
+                    outfile_score.write(f'>{read_title}\n')
                 else:
-                    outfile_score.write('>%s\n' % line[1:])
+                    outfile_score.write(f'>{line[1:]}\n')
             else:
                 # fourth line is quality scores
                 qual = ''
@@ -85,10 +85,10 @@ def __main__():
                         quality_score_startswith = default_coding_value
                     else:
                         stop_err('Invalid fastqsolexa format at line %d: the number of quality scores ( %d ) is not the same as bases ( %d ).' % (i + 1, quality_score_length, read_length))
-                    for j, char in enumerate(line):
+                    for char in line:
                         score = ord(char) - quality_score_startswith    # 64
-                        qual = "{}{} ".format(qual, str(score))
-                outfile_score.write('%s\n' % qual)
+                        qual = f"{qual}{str(score)} "
+                outfile_score.write(f'{qual}\n')
 
 
 if __name__ == "__main__":
