@@ -4,6 +4,7 @@ MetaCyto analysis datatypes.
 
 import logging
 
+from galaxy.datatypes.sniff import FilePrefix
 from galaxy.datatypes.tabular import Tabular
 
 log = logging.getLogger(__name__)
@@ -13,7 +14,7 @@ class mStats(Tabular):
     """Class describing the table of cluster statistics output from MetaCyto"""
     file_ext = "metacyto_stats.txt"
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """Quick test on file headings"""
         if file_prefix.startswith("fcs_files\tcluster_id\tlabel\tfcs_names"):
             header_line = file_prefix.string_io().readline()
@@ -28,5 +29,5 @@ class mSummary(Tabular):
     """Class describing the summary table output by MetaCyto after FCS preprocessing"""
     file_ext = "metacyto_summary.txt"
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         return file_prefix.startswith('study_id\tantibodies\tfilenames')
