@@ -6,7 +6,7 @@ import imp
 import logging
 import os
 from string import Template
-from typing import Dict
+from typing import Dict, List, Optional, Tuple, TYPE_CHECKING
 
 import yaml
 
@@ -28,6 +28,9 @@ from . import (
 )
 from .display_applications.application import DisplayApplication
 
+if TYPE_CHECKING:
+    from galaxy.model import DatasetInstance
+
 
 class ConfigurationError(Exception):
     pass
@@ -357,6 +360,8 @@ class Registry:
                                         "description": description,
                                         "description_url": description_url,
                                     })
+                                    if auto_compressed_type == 'gz':
+                                        self.converters.append((f"uncompressed_to_{auto_compressed_type}.xml", extension, compressed_extension))
                                     self.converters.append((f"{auto_compressed_type}_to_uncompressed.xml", compressed_extension, extension))
                                     if datatype_class not in compressed_sniffers:
                                         compressed_sniffers[datatype_class] = []
@@ -871,7 +876,7 @@ class Registry:
             for ext2, converters_dict in self.datatype_converters.items():
                 converter_datatype = type(self.get_datatype_by_extension(ext2))
                 if issubclass(source_datatype, converter_datatype):
-                    converters.update(converters_dict)
+                    converters.update({k: v for k, v in converters_dict.items() if k != ext})
             # Ensure ext-level converters are present
             if ext in self.datatype_converters.keys():
                 converters.update(self.datatype_converters[ext])
@@ -885,7 +890,9 @@ class Registry:
             return converters[target_ext]
         return None
 
-    def find_conversion_destination_for_dataset_by_extensions(self, dataset_or_ext, accepted_formats, converter_safe=True):
+    def find_conversion_destination_for_dataset_by_extensions(
+        self, dataset_or_ext, accepted_formats: List[str], converter_safe: bool = True
+    ) -> Tuple[bool, Optional[str], Optional["DatasetInstance"]]:
         """
         returns (direct_match, converted_ext, converted_dataset)
         - direct match is True iff no the data set already has an accepted format
