@@ -25,6 +25,7 @@ from galaxy.datatypes.metadata import (
 )
 from galaxy.datatypes.sniff import (
     build_sniff_from_prefix,
+    FilePrefix,
     get_headers,
     iter_headers,
     validate_tabular,
@@ -34,6 +35,8 @@ from . import dataproviders
 
 log = logging.getLogger(__name__)
 
+MAX_DATA_LINES = 100000
+
 
 @dataproviders.decorators.has_dataproviders
 class TabularData(data.Text):
@@ -44,7 +47,6 @@ class TabularData(data.Text):
     data_line_offset = 0
     max_peek_columns = 50
 
-    """Add metadata elements"""
     MetadataElement(name="comment_lines", default=0, desc="Number of comment lines", readonly=False, optional=True, no_value=0)
     MetadataElement(name="data_lines", default=0, desc="Number of data lines", readonly=True, visible=False, optional=True, no_value=0)
     MetadataElement(name="columns", default=0, desc="Number of columns", readonly=True, visible=False, no_value=0)
@@ -56,7 +58,7 @@ class TabularData(data.Text):
     def set_meta(self, dataset, **kwd):
         raise NotImplementedError
 
-    def set_peek(self, dataset, line_count=None, is_multi_byte=False, WIDTH=256, skipchars=None, line_wrap=False, **kwd):
+    def set_peek(self, dataset, line_count=None, WIDTH=256, skipchars=None, line_wrap=False, **kwd):
         super().set_peek(dataset, line_count=line_count, WIDTH=WIDTH, skipchars=skipchars, line_wrap=line_wrap)
         if dataset.metadata.comment_lines:
             dataset.blurb = f"{dataset.blurb}, {util.commaify(str(dataset.metadata.comment_lines))} comments"
@@ -86,25 +88,26 @@ class TabularData(data.Text):
                       })
 
     def display_data(self, trans, dataset, preview=False, filename=None, to_ext=None, offset=None, ck_size=None, **kwd):
+        headers = kwd.get("headers", {})
         preview = util.string_as_bool(preview)
         if offset is not None:
-            return self.get_chunk(trans, dataset, offset, ck_size)
+            return self.get_chunk(trans, dataset, offset, ck_size), headers
         elif to_ext or not preview:
             to_ext = to_ext or dataset.extension
-            return self._serve_raw(trans, dataset, to_ext, **kwd)
+            return self._serve_raw(dataset, to_ext, headers, **kwd)
         elif dataset.metadata.columns > 100:
             # Fancy tabular display is only suitable for datasets without an incredibly large number of columns.
             # We should add a new datatype 'matrix', with its own draw method, suitable for this kind of data.
             # For now, default to the old behavior, ugly as it is.  Remove this after adding 'matrix'.
             max_peek_size = 1000000  # 1 MB
             if os.stat(dataset.file_name).st_size < max_peek_size:
-                self._clean_and_set_mime_type(trans, dataset.get_mime())
-                return open(dataset.file_name, mode='rb')
+                self._clean_and_set_mime_type(trans, dataset.get_mime(), headers)
+                return open(dataset.file_name, mode='rb'), headers
             else:
-                trans.response.set_content_type("text/html")
-                return trans.stream_template_mako("/dataset/large_file.mako",
-                                                  truncated_data=open(dataset.file_name, mode='r').read(max_peek_size),
-                                                  data=dataset)
+                headers["content-type"] = "text/html"
+                return trans.fill_template_mako("/dataset/large_file.mako",
+                                                truncated_data=open(dataset.file_name).read(max_peek_size),
+                                                data=dataset), headers
         else:
             column_names = 'null'
             if dataset.metadata.column_names:
@@ -122,7 +125,7 @@ class TabularData(data.Text):
                                        chunk=self.get_chunk(trans, dataset, 0),
                                        column_number=column_number,
                                        column_names=column_names,
-                                       column_types=column_types)
+                                       column_types=column_types), headers
 
     def display_as_markdown(self, dataset_instance, markdown_format_helpers):
         with open(dataset_instance.file_name) as f:
@@ -269,7 +272,7 @@ class Tabular(TabularData):
     def get_column_names(self, first_line=None):
         return None
 
-    def set_meta(self, dataset, overwrite=True, skip=None, max_data_lines=100000, max_guess_type_data_lines=None, **kwd):
+    def set_meta(self, dataset, overwrite=True, skip=None, max_data_lines=MAX_DATA_LINES, max_guess_type_data_lines=None, **kwd):
         """
         Tries to determine the number of columns as well as those columns that
         contain numerical values in the dataset.  A skip parameter is used
@@ -366,10 +369,7 @@ class Tabular(TabularData):
             # NOTE: if skip > num_check_lines, we won't detect any metadata, and will use default
             with compression_utils.get_fileobj(dataset.file_name) as dataset_fh:
                 i = 0
-                while True:
-                    line = dataset_fh.readline()
-                    if not line:
-                        break
+                for line in iter(dataset_fh.readline, ''):
                     line = line.rstrip('\r\n')
                     if i == 0:
                         column_names = self.get_column_names(first_line=line)
@@ -476,8 +476,8 @@ class Sam(Tabular, _BamOrSam):
     track_type = "ReadTrack"
     data_sources = {"data": "bam", "index": "bigwig"}
 
-    MetadataElement(name="bam_version", default=None, desc="BAM Version", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=None)
-    MetadataElement(name="sort_order", default=None, desc="Sort Order", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=None)
+    MetadataElement(name="bam_version", default=None, desc="BAM Version", param=MetadataParameter, readonly=True, visible=False, optional=True)
+    MetadataElement(name="sort_order", default=None, desc="Sort Order", param=MetadataParameter, readonly=True, visible=False, optional=True)
     MetadataElement(name="read_groups", default=[], desc="Read Groups", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=[])
     MetadataElement(name="reference_names", default=[], desc="Chromosome Names", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=[])
     MetadataElement(name="reference_lengths", default=[], desc="Chromosome Lengths", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=[])
@@ -494,7 +494,7 @@ class Sam(Tabular, _BamOrSam):
         """Returns formated html of peek"""
         return self.make_html_table(dataset, column_names=self.column_names)
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         Determines whether the file is in SAM format
 
@@ -523,13 +523,9 @@ class Sam(Tabular, _BamOrSam):
         >>> Sam().sniff( fname )
         True
         """
-        fh = file_prefix.string_io()
         count = 0
-        while True:
-            line = fh.readline()
+        for line in file_prefix.line_iterator():
             line = line.strip()
-            if not line:
-                break  # EOF
             if line:
                 if line[0] != '@':
                     line_pieces = line.split('\t')
@@ -681,7 +677,6 @@ class Pileup(Tabular):
     line_class = "genomic coordinate"
     data_sources = {"data": "tabix"}
 
-    """Add metadata elements"""
     MetadataElement(name="chromCol", default=1, desc="Chrom column", param=metadata.ColumnParameter)
     MetadataElement(name="startCol", default=2, desc="Start column", param=metadata.ColumnParameter)
     MetadataElement(name="endCol", default=2, desc="End column", param=metadata.ColumnParameter)
@@ -698,7 +693,7 @@ class Pileup(Tabular):
         """Return options for removing errors along with a description"""
         return [("lines", "Remove erroneous lines")]
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         Checks for 'pileup-ness'
 
@@ -831,7 +826,7 @@ class BaseVcf(Tabular):
 class Vcf(BaseVcf):
     file_ext = 'vcf'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         return self._sniff(file_prefix)
 
 
@@ -842,7 +837,7 @@ class VcfGz(BaseVcf, binary.Binary):
     compressed = True
     compressed_format = "gzip"
 
-    MetadataElement(name="tabix_index", desc="Vcf Index File", param=metadata.FileParameter, file_ext="tbi", readonly=True, no_value=None, visible=False, optional=True)
+    MetadataElement(name="tabix_index", desc="Vcf Index File", param=metadata.FileParameter, file_ext="tbi", readonly=True, visible=False, optional=True)
 
     def sniff(self, filename):
         if not self._sniff(filename):
@@ -857,7 +852,7 @@ class VcfGz(BaseVcf, binary.Binary):
 
     def set_meta(self, dataset, **kwd):
         super().set_meta(dataset, **kwd)
-        """ Creates the index for the VCF file. """
+        # Creates the index for the VCF file.
         # These metadata values are not accessible by users, always overwrite
         index_file = dataset.metadata.tabix_index
         if not index_file:
@@ -915,7 +910,7 @@ class Eland(Tabular):
             out = f"Can't create peek {exc}"
         return out
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         Determines whether the file is in ELAND export format
 
@@ -928,13 +923,11 @@ class Eland(Tabular):
             - LANE, TILEm X, Y, INDEX, READ_NO, SEQ, QUAL, POSITION, *STRAND, FILT must be correct
             - We will only check that up to the first 5 alignments are correctly formatted.
         """
-        fh = file_prefix.string_io()
         count = 0
-        while True:
-            line = fh.readline()
+        for line in file_prefix.line_iterator():
             line = line.strip()
             if not line:
-                break  # EOF
+                break  # Had a EOF comment previously, but this does not indicate EOF. I assume empty lines are not valid and this was intentional.
             if line:
                 line_pieces = line.split('\t')
                 if len(line_pieces) != 22:
@@ -990,7 +983,7 @@ class Eland(Tabular):
 class ElandMulti(Tabular):
     file_ext = 'elandmulti'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         return False
 
 
@@ -1086,16 +1079,14 @@ class BaseCSV(TabularData):
             return False
         if (auto_dialect.quotechar != self.dialect.quotechar):
             return False
-        """
-        Not checking for other dialect options
-        They may be mis detected from just the sample.
-        Or not effect the read such as doublequote
+        # Not checking for other dialect options
+        # They may be mis detected from just the sample.
+        # Or not effect the read such as doublequote
 
-        Optional: Check for headers as in the past.
-        Note No way around Python's csv calling Sniffer.sniff again.
-        Note Without checking the dialect returned by sniff
-              this test may be checking the wrong dialect.
-        """
+        # Optional: Check for headers as in the past.
+        # Note: No way around Python's csv calling Sniffer.sniff again.
+        # Note: Without checking the dialect returned by sniff
+        #       this test may be checking the wrong dialect.
         if not csv.Sniffer().has_header(big_peek):
             return False
         return True
@@ -1187,7 +1178,7 @@ class ConnectivityTable(Tabular):
 
         dataset.metadata.data_lines = data_lines
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         The ConnectivityTable (CT) is a file format used for describing
         RNA 2D structures by tools including MFOLD, UNAFOLD and
@@ -1310,7 +1301,7 @@ class MatrixMarket(TabularData):
     def __init__(self, **kwd):
         super().__init__(**kwd)
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         return file_prefix.startswith('%%MatrixMarket matrix coordinate')
 
     def set_meta(self, dataset, overwrite=True, skip=None, max_data_lines=5, **kwd):
@@ -1342,13 +1333,6 @@ class MatrixMarket(TabularData):
 
 @build_sniff_from_prefix
 class CMAP(TabularData):
-    MetadataElement(name="cmap_version", default='0.2', desc="version of cmap", readonly=True, visible=True, optional=False, no_value='0.2')
-    MetadataElement(name="label_channels", default=1, desc="the number of label channels", readonly=True, visible=True, optional=False, no_value=1)
-    MetadataElement(name="nickase_recognition_site_1", default=[], desc="comma separated list of label motif recognition sequences for channel 1", readonly=True, visible=True, optional=False, no_value=[])
-    MetadataElement(name="number_of_consensus_nanomaps", default=0, desc="the total number of consensus genome maps in the CMAP file", readonly=True, visible=True, optional=False, no_value=0)
-    MetadataElement(name="nickase_recognition_site_2", default=[], desc="comma separated list of label motif recognition sequences for channel 2", readonly=True, visible=True, optional=True, no_value=[])
-    MetadataElement(name="channel_1_color", default=[], desc="channel 1 color", readonly=True, visible=True, optional=True, no_value=[])
-    MetadataElement(name="channel_2_color", default=[], desc="channel 2 color", readonly=True, visible=True, optional=True, no_value=[])
     """
     # CMAP File Version:    2.0
     # Label Channels:   1
@@ -1364,7 +1348,15 @@ class CMAP(TabularData):
     """
     file_ext = "cmap"
 
-    def sniff_prefix(self, file_prefix):
+    MetadataElement(name="cmap_version", default='0.2', desc="version of cmap", readonly=True, visible=True, optional=False, no_value='0.2')
+    MetadataElement(name="label_channels", default=1, desc="the number of label channels", readonly=True, visible=True, optional=False, no_value=1)
+    MetadataElement(name="nickase_recognition_site_1", default=[], desc="comma separated list of label motif recognition sequences for channel 1", readonly=True, visible=True, optional=False, no_value=[])
+    MetadataElement(name="number_of_consensus_nanomaps", default=0, desc="the total number of consensus genome maps in the CMAP file", readonly=True, visible=True, optional=False, no_value=0)
+    MetadataElement(name="nickase_recognition_site_2", default=[], desc="comma separated list of label motif recognition sequences for channel 2", readonly=True, visible=True, optional=True, no_value=[])
+    MetadataElement(name="channel_1_color", default=[], desc="channel 1 color", readonly=True, visible=True, optional=True, no_value=[])
+    MetadataElement(name="channel_2_color", default=[], desc="channel 2 color", readonly=True, visible=True, optional=True, no_value=[])
+
+    def sniff_prefix(self, file_prefix: FilePrefix):
         handle = file_prefix.string_io()
         for line in handle:
             if not line.startswith('#'):
