@@ -12,9 +12,13 @@ import tempfile
 
 import yaml
 
-from galaxy.datatypes.data import get_file_peek, Text
+from galaxy.datatypes.data import get_file_peek, Headers, Text
 from galaxy.datatypes.metadata import MetadataElement, MetadataParameter
-from galaxy.datatypes.sniff import build_sniff_from_prefix, iter_headers
+from galaxy.datatypes.sniff import (
+    build_sniff_from_prefix,
+    FilePrefix,
+    iter_headers,
+)
 from galaxy.util import (
     nice_size,
     string_as_bool,
@@ -30,7 +34,7 @@ class Html(Text):
     edam_format = "format_2331"
     file_ext = "html"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         if not dataset.dataset.purged:
             dataset.peek = "HTML file"
             dataset.blurb = nice_size(dataset.get_size())
@@ -42,7 +46,7 @@ class Html(Text):
         """Returns the mime type of the datatype"""
         return 'text/html'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         Determines whether the file is in html format
 
@@ -66,7 +70,7 @@ class Json(Text):
     edam_format = "format_3464"
     file_ext = "json"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         if not dataset.dataset.purged:
             dataset.peek = get_file_peek(dataset.file_name)
             dataset.blurb = "JavaScript Object Notation (JSON)"
@@ -78,7 +82,7 @@ class Json(Text):
         """Returns the mime type of the datatype"""
         return 'application/json'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
             Try to load the string with the json module. If successful it's a json file.
         """
@@ -114,7 +118,7 @@ class ExpressionJson(Json):
     """ Represents the non-data input or output to a tool or workflow.
     """
     file_ext = "json"
-    MetadataElement(name="json_type", default=None, desc="JavaScript or JSON type of expression", readonly=True, visible=True, no_value=None)
+    MetadataElement(name="json_type", default=None, desc="JavaScript or JSON type of expression", readonly=True, visible=True)
 
     def set_meta(self, dataset, **kwd):
         """
@@ -144,7 +148,7 @@ class ExpressionJson(Json):
 class Ipynb(Json):
     file_ext = "ipynb"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         if not dataset.dataset.purged:
             dataset.peek = get_file_peek(dataset.file_name)
             dataset.blurb = "Jupyter Notebook"
@@ -152,7 +156,7 @@ class Ipynb(Json):
             dataset.peek = 'file does not exist'
             dataset.blurb = 'file purged from disc'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
             Try to load the string with the json module. If successful it's a json file.
         """
@@ -168,17 +172,19 @@ class Ipynb(Json):
                 return False
 
     def display_data(self, trans, dataset, preview=False, filename=None, to_ext=None, **kwd):
+        headers = kwd.get("headers", {})
         config = trans.app.config
         trust = getattr(config, 'trust_jupyter_notebook_conversion', False)
         if trust:
-            return self._display_data_trusted(trans, dataset, preview=preview, filename=filename, to_ext=to_ext, **kwd)
+            return self._display_data_trusted(trans, dataset, preview=preview, filename=filename, to_ext=to_ext, headers=headers, **kwd)
         else:
-            return super().display_data(trans, dataset, preview=preview, filename=filename, to_ext=to_ext, **kwd)
+            return super().display_data(trans, dataset, preview=preview, filename=filename, to_ext=to_ext, headers=headers, **kwd)
 
     def _display_data_trusted(self, trans, dataset, preview=False, filename=None, to_ext=None, **kwd):
+        headers = kwd.get("headers", {})
         preview = string_as_bool(preview)
         if to_ext or not preview:
-            return self._serve_raw(trans, dataset, to_ext, **kwd)
+            return self._serve_raw(dataset, to_ext, headers, **kwd)
         else:
             with tempfile.NamedTemporaryFile(delete=False) as ofile_handle:
                 ofilename = ofile_handle.name
@@ -189,7 +195,7 @@ class Ipynb(Json):
             except subprocess.CalledProcessError:
                 ofilename = dataset.file_name
                 log.exception('Command "%s" failed. Could not convert the Jupyter Notebook to HTML, defaulting to plain text.', ' '.join(map(shlex.quote, cmd)))
-            return open(ofilename, mode='rb')
+            return open(ofilename, mode='rb'), headers
 
     def set_meta(self, dataset, **kwd):
         """
@@ -215,22 +221,22 @@ class Biom1(Json):
     MetadataElement(name="table_format_url", default="", desc="table_format_url", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value="")
     MetadataElement(name="table_date", default="", desc="table_date", param=MetadataParameter, readonly=True, visible=True, optional=True, no_value="")
     MetadataElement(name="table_type", default="", desc="table_type", param=MetadataParameter, readonly=True, visible=True, optional=True, no_value="")
-    MetadataElement(name="table_id", default=None, desc="table_id", param=MetadataParameter, readonly=True, visible=True, optional=True, no_value=None)
+    MetadataElement(name="table_id", default=None, desc="table_id", param=MetadataParameter, readonly=True, visible=True, optional=True)
     MetadataElement(name="table_columns", default=[], desc="table_columns", param=MetadataParameter, readonly=True, visible=False, optional=True, no_value=[])
     MetadataElement(name="table_column_metadata_headers", default=[], desc="table_column_metadata_headers", param=MetadataParameter, readonly=True, visible=True, optional=True, no_value=[])
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         super().set_peek(dataset)
         if not dataset.dataset.purged:
             dataset.blurb = "Biological Observation Matrix v1"
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         is_biom = False
         if self._looks_like_json(file_prefix):
             is_biom = self._looks_like_biom(file_prefix)
         return is_biom
 
-    def _looks_like_biom(self, file_prefix, load_size=50000):
+    def _looks_like_biom(self, file_prefix: FilePrefix, load_size=50000):
         """
         @param filepath: [str] The path to the evaluated file.
         @param load_size: [int] The size of the file block load in RAM (in
@@ -304,22 +310,23 @@ class Biom1(Json):
 
 @build_sniff_from_prefix
 class ImgtJson(Json):
+    """
+    https://github.com/repseqio/library-imgt/releases
+    Data coming from IMGT server may be used for academic research only,
+    provided that it is referred to IMGT®, and cited as:
+    "IMGT®, the international ImMunoGeneTics information system®
+    http://www.imgt.org (founder and director: Marie-Paule Lefranc, Montpellier, France)."
+    """
     file_ext = "imgt.json"
-    MetadataElement(name="taxon_names", default=[], desc="taxonID: names", readonly=True, visible=True, no_value=[])
-    """
-        https://github.com/repseqio/library-imgt/releases
-        Data coming from IMGT server may be used for academic research only,
-        provided that it is referred to IMGT®, and cited as:
-        "IMGT®, the international ImMunoGeneTics information system®
-        http://www.imgt.org (founder and director: Marie-Paule Lefranc, Montpellier, France)."
-    """
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    MetadataElement(name="taxon_names", default=[], desc="taxonID: names", readonly=True, visible=True, no_value=[])
+
+    def set_peek(self, dataset):
         super().set_peek(dataset)
         if not dataset.dataset.purged:
             dataset.blurb = "IMGT Library"
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         Determines whether the file is in json format with imgt elements
 
@@ -336,7 +343,7 @@ class ImgtJson(Json):
             is_imgt = self._looks_like_imgt(file_prefix)
         return is_imgt
 
-    def _looks_like_imgt(self, file_prefix, load_size=5000):
+    def _looks_like_imgt(self, file_prefix: FilePrefix, load_size=5000):
         """
         @param filepath: [str] The path to the evaluated file.
         @param load_size: [int] The size of the file block load in RAM (in
@@ -379,12 +386,12 @@ class GeoJson(Json):
     """
     file_ext = "geojson"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         super().set_peek(dataset)
         if not dataset.dataset.purged:
             dataset.blurb = "GeoJSON"
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         Determines whether the file is in json format with imgt elements
 
@@ -401,7 +408,7 @@ class GeoJson(Json):
             is_geojson = self._looks_like_geojson(file_prefix)
         return is_geojson
 
-    def _looks_like_geojson(self, file_prefix, load_size=5000):
+    def _looks_like_geojson(self, file_prefix: FilePrefix, load_size=5000):
         """
         One of "Point", "MultiPoint", "LineString", "MultiLineString", "Polygon", "MultiPolygon", and "GeometryCollection" needs to be present.
         All of "type", "geometry", and "coordinates" needs to be present.
@@ -428,7 +435,7 @@ class Obo(Text):
     edam_format = "format_2549"
     file_ext = "obo"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         if not dataset.dataset.purged:
             dataset.peek = get_file_peek(dataset.file_name)
             dataset.blurb = "Open Biomedical Ontology (OBO)"
@@ -436,7 +443,7 @@ class Obo(Text):
             dataset.peek = 'file does not exist'
             dataset.blurb = 'file purged from disc'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
             Try to guess the Obo filetype.
             It usually starts with a "format-version:" string and has several stanzas which starts with "id:".
@@ -458,17 +465,16 @@ class Obo(Text):
 @build_sniff_from_prefix
 class Arff(Text):
     """
-        An ARFF (Attribute-Relation File Format) file is an ASCII text file that describes a list of instances sharing a set of attributes.
-        http://weka.wikispaces.com/ARFF
+    An ARFF (Attribute-Relation File Format) file is an ASCII text file that describes a list of instances sharing a set of attributes.
+    http://weka.wikispaces.com/ARFF
     """
     edam_format = "format_3581"
     file_ext = "arff"
 
-    """Add metadata elements"""
     MetadataElement(name="comment_lines", default=0, desc="Number of comment lines", readonly=True, optional=True, no_value=0)
     MetadataElement(name="columns", default=0, desc="Number of columns", readonly=True, visible=True, no_value=0)
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         if not dataset.dataset.purged:
             dataset.peek = get_file_peek(dataset.file_name)
             dataset.blurb = "Attribute-Relation File Format (ARFF)"
@@ -477,7 +483,7 @@ class Arff(Text):
             dataset.peek = 'file does not exist'
             dataset.blurb = 'file purged from disc'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
             Try to guess the Arff filetype.
             It usually starts with a "format-version:" string and has several stanzas which starts with "id:".
@@ -562,8 +568,8 @@ class SnpEffDb(Text):
     """Class describing a SnpEff genome build"""
     edam_format = "format_3624"
     file_ext = "snpeffdb"
-    MetadataElement(name="genome_version", default=None, desc="Genome Version", readonly=True, visible=True, no_value=None)
-    MetadataElement(name="snpeff_version", default="SnpEff4.0", desc="SnpEff Version", readonly=True, visible=True, no_value=None)
+    MetadataElement(name="genome_version", default=None, desc="Genome Version", readonly=True, visible=True)
+    MetadataElement(name="snpeff_version", default="SnpEff4.0", desc="SnpEff Version", readonly=True, visible=True)
     MetadataElement(name="regulation", default=[], desc="Regulation Names", readonly=True, visible=True, no_value=[], optional=True)
     MetadataElement(name="annotation", default=[], desc="Annotation Names", readonly=True, visible=True, no_value=[], optional=True)
 
@@ -630,24 +636,27 @@ class SnpEffDb(Text):
 
 
 class SnpSiftDbNSFP(Text):
-    """Class describing a dbNSFP database prepared fpr use by SnpSift dbnsfp """
-    MetadataElement(name='reference_name', default='dbSNFP', desc='Reference Name', readonly=True, visible=True, set_in_upload=True, no_value='dbSNFP')
-    MetadataElement(name="bgzip", default=None, desc="dbNSFP bgzip", readonly=True, visible=True, no_value=None)
-    MetadataElement(name="index", default=None, desc="Tabix Index File", readonly=True, visible=True, no_value=None)
-    MetadataElement(name="annotation", default=[], desc="Annotation Names", readonly=True, visible=True, no_value=[])
+    """
+    Class describing a dbNSFP database prepared fpr use by SnpSift dbnsfp
+
+    The dbNSFP file is a tabular file with 1 header line.
+    The first 4 columns are required to be: chrom	pos	ref	alt
+    These match columns 1,2,4,5 of the VCF file
+    SnpSift requires the file to be block-gzipped and the indexed with samtools tabix
+
+    Example:
+    - Compress using block-gzip algorithm:
+    $ bgzip dbNSFP2.3.txt
+    - Create tabix index
+    $ tabix -s 1 -b 2 -e 2 dbNSFP2.3.txt.gz
+    """
     file_ext = "snpsiftdbnsfp"
     composite_type = 'auto_primary_file'
-    """
-    ## The dbNSFP file is a tabular file with 1 header line
-    ## The first 4 columns are required to be: chrom	pos	ref	alt
-    ## These match columns 1,2,4,5 of the VCF file
-    ## SnpSift requires the file to be block-gzipped and the indexed with samtools tabix
-    ## Example:
-    ## Compress using block-gzip algorithm
-    bgzip dbNSFP2.3.txt
-    ## Create tabix index
-    tabix -s 1 -b 2 -e 2 dbNSFP2.3.txt.gz
-    """
+
+    MetadataElement(name='reference_name', default='dbSNFP', desc='Reference Name', readonly=True, visible=True, set_in_upload=True, no_value='dbSNFP')
+    MetadataElement(name="bgzip", default=None, desc="dbNSFP bgzip", readonly=True, visible=True)
+    MetadataElement(name="index", default=None, desc="Tabix Index File", readonly=True, visible=True)
+    MetadataElement(name="annotation", default=[], desc="Annotation Names", readonly=True, visible=True, no_value=[])
 
     def __init__(self, **kwd):
         super().__init__(**kwd)
@@ -695,7 +704,7 @@ class SnpSiftDbNSFP(Text):
         except Exception as e:
             log.warning("set_meta fname: %s  %s", dataset.file_name if dataset and dataset.file_name else 'Unkwown', unicodify(e))
 
-        def set_peek(self, dataset, is_multi_byte=False):
+        def set_peek(self, dataset):
             if not dataset.dataset.purged:
                 dataset.peek = f"{dataset.metadata.reference_name} :  {','.join(dataset.metadata.annotation)}"
                 dataset.blurb = f'{dataset.metadata.reference_name}'
@@ -709,7 +718,7 @@ class IQTree(Text):
     """IQ-TREE format"""
     file_ext = 'iqtree'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         Detect the IQTree file
 
@@ -741,7 +750,7 @@ class Paf(Text):
     """
     file_ext = "paf"
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         >>> from galaxy.datatypes.sniff import get_test_fname
         >>> fname = get_test_fname('A-3105.paf')
@@ -775,7 +784,7 @@ class Gfa1(Text):
     """
     file_ext = "gfa1"
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         >>> from galaxy.datatypes.sniff import get_test_fname
         >>> fname = get_test_fname('big.gfa1')
@@ -824,7 +833,7 @@ class Gfa2(Text):
     """
     file_ext = "gfa2"
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         >>> from galaxy.datatypes.sniff import get_test_fname
         >>> fname = get_test_fname('sample.gfa2')
@@ -865,7 +874,7 @@ class Yaml(Text):
     """Yaml files"""
     file_ext = "yaml"
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
             Try to load the string with the yaml module. If successful it's a yaml file.
         """
@@ -875,7 +884,14 @@ class Yaml(Text):
         """Returns the mime type of the datatype"""
         return 'application/yaml'
 
-    def _looks_like_yaml(self, file_prefix):
+    def _yield_user_file_content(self, trans, from_dataset, filename, headers: Headers):
+        # Override non-standard application/yaml mediatype with
+        # text/plain, so preview is shown in preview iframe,
+        # instead of downloading the file.
+        headers["content-type"] = "text/plain"
+        return super()._yield_user_file_content(trans, from_dataset, filename, headers)
+
+    def _looks_like_yaml(self, file_prefix: FilePrefix):
         # Pattern used by SequenceSplitLocations
         if file_prefix.file_size < 50000 and not file_prefix.truncated:
             # If the file is small enough - don't guess just check.
