@@ -219,20 +219,16 @@ class GFFReaderWrapper(NiceReaderWrapper):
         while True:
             try:
                 interval = super(GenomicIntervalReader, self).__next__()
-                raw_size += len(self.current_line)
             except StopIteration:
                 # No more intervals to read, but last feature needs to be
                 # returned.
                 interval = None
-                raw_size += len(self.current_line)
                 break
             except ParseError as e:
                 handle_parse_error(e)
-                raw_size += len(self.current_line)
                 continue
-            # TODO: When no longer supporting python 2.4 use finally:
-            # finally:
-            # raw_size += len( self.current_line )
+            finally:
+                raw_size += len(self.current_line)
 
             # Ignore comments.
             if isinstance(interval, Comment):
@@ -369,7 +365,10 @@ def parse_gff3_attributes(attr_str):
     attributes_list = attr_str.split(";")
     attributes = {}
     for tag_value_pair in attributes_list:
-        pair = tag_value_pair.strip().split("=")
+        tag_value_pair = tag_value_pair.strip()
+        if tag_value_pair == '':
+            continue
+        pair = tag_value_pair.split("=")
         if len(pair) == 1:
             raise Exception(f"Attribute '{tag_value_pair}' does not contain a '='")
         if pair == '':
