@@ -6,10 +6,14 @@ import re
 
 from galaxy import util
 from galaxy.datatypes.metadata import MetadataElement
+from galaxy.datatypes.sniff import (
+    build_sniff_from_prefix,
+    disable_parent_class_sniffing,
+    FilePrefix,
+)
 from . import (
     data,
     dataproviders,
-    sniff
 )
 
 log = logging.getLogger(__name__)
@@ -19,13 +23,13 @@ SBML_MARKER = re.compile(r'\<sbml')
 
 
 @dataproviders.decorators.has_dataproviders
-@sniff.build_sniff_from_prefix
+@build_sniff_from_prefix
 class GenericXml(data.Text):
     """Base format class for any XML file."""
     edam_format = "format_2332"
     file_ext = "xml"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
             dataset.peek = data.get_file_peek(dataset.file_name)
@@ -34,17 +38,15 @@ class GenericXml(data.Text):
             dataset.peek = 'file does not exist'
             dataset.blurb = 'file purged from disk'
 
-    def _has_root_element_in_prefix(self, file_prefix, root):
-        contents = file_prefix.string_io()
-        while True:
-            line = contents.readline()
-            if line is None or not line.startswith('<?'):
+    def _has_root_element_in_prefix(self, file_prefix: FilePrefix, root):
+        for line in file_prefix.line_iterator():
+            if not line.startswith('<?'):
                 break
         # pattern match <root or <ns:root for any ns string
         pattern = r'^<(\w*:)?%s' % root
-        return line is not None and re.match(pattern, line) is not None
+        return re.match(pattern, line) is not None
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         Determines whether the file is XML or not
 
@@ -72,12 +74,12 @@ class GenericXml(data.Text):
         return dataproviders.hierarchy.XMLDataProvider(dataset_source, **settings)
 
 
-@sniff.disable_parent_class_sniffing
+@disable_parent_class_sniffing
 class MEMEXml(GenericXml):
     """MEME XML Output data"""
     file_ext = "memexml"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
             dataset.peek = data.get_file_peek(dataset.file_name)
@@ -87,12 +89,12 @@ class MEMEXml(GenericXml):
             dataset.blurb = 'file purged from disk'
 
 
-@sniff.disable_parent_class_sniffing
+@disable_parent_class_sniffing
 class CisML(GenericXml):
     """CisML XML data"""  # see: http://www.ncbi.nlm.nih.gov/pubmed/15001475
     file_ext = "cisml"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
             dataset.peek = data.get_file_peek(dataset.file_name)
@@ -110,15 +112,15 @@ class Dzi(GenericXml):
 
     # General elements.
     MetadataElement(name="base_name", desc="Base name for this dataset", default='DeepZoomImage', readonly=True, set_in_upload=True)
-    MetadataElement(name="format", desc="File format of the tiles", default=None, readonly=True, visible=True, no_value=None)
-    MetadataElement(name="tile_size", desc="Size of tiles", default=None, readonly=True, visible=True, no_value=None)
+    MetadataElement(name="format", desc="File format of the tiles", default=None, readonly=True, visible=True)
+    MetadataElement(name="tile_size", desc="Size of tiles", default=None, readonly=True, visible=True)
     # Collection elements.
-    MetadataElement(name="max_level", desc="Max pyramid level", default=None, readonly=True, optional=True, visible=True, no_value=None)
-    MetadataElement(name="quality", desc="Quality", default=None, readonly=True, optional=True, visible=True, no_value=None)
+    MetadataElement(name="max_level", desc="Max pyramid level", default=None, readonly=True, optional=True, visible=True)
+    MetadataElement(name="quality", desc="Quality", default=None, readonly=True, optional=True, visible=True)
     # Image elements.
-    MetadataElement(name="height", desc="Size height", default=None, readonly=True, optional=True, visible=True, no_value=None)
-    MetadataElement(name="overlap", desc="Overlap of all four sides of tiles", default=None, readonly=True, optional=True, visible=True, no_value=None)
-    MetadataElement(name="width", desc="Size width", default=None, readonly=True, optional=True, visible=True, no_value=None)
+    MetadataElement(name="height", desc="Size height", default=None, readonly=True, optional=True, visible=True)
+    MetadataElement(name="overlap", desc="Overlap of all four sides of tiles", default=None, readonly=True, optional=True, visible=True)
+    MetadataElement(name="width", desc="Size width", default=None, readonly=True, optional=True, visible=True)
 
     file_ext = 'dzi'
 
@@ -146,7 +148,7 @@ class Dzi(GenericXml):
         """ Returns a list of visualizations for datatype"""
         return ['openseadragon']
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         if not dataset.dataset.purged:
             dataset.peek = data.get_file_peek(dataset.file_name)
             dataset.blurb = "Deep Zoom Image"
@@ -154,7 +156,7 @@ class Dzi(GenericXml):
             dataset.peek = 'file does not exist'
             dataset.blurb = 'file purged from disc'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
         Checking for keyword - 'Collection' or 'Image' in the first 200 lines.
         >>> from galaxy.datatypes.sniff import get_test_fname
@@ -178,7 +180,7 @@ class Phyloxml(GenericXml):
     edam_format = "format_3159"
     file_ext = "phyloxml"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
             dataset.peek = data.get_file_peek(dataset.file_name)
@@ -187,7 +189,7 @@ class Phyloxml(GenericXml):
             dataset.peek = 'file does not exist'
             dataset.blurb = 'file purged from disk'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """"Checking for keyword - 'phyloxml' always in lowercase in the first few lines.
 
         >>> from galaxy.datatypes.sniff import get_test_fname
@@ -219,7 +221,7 @@ class Owl(GenericXml):
     edam_format = "format_3262"
     file_ext = "owl"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         if not dataset.dataset.purged:
             dataset.peek = data.get_file_peek(dataset.file_name)
             dataset.blurb = "Web Ontology Language OWL"
@@ -227,7 +229,7 @@ class Owl(GenericXml):
             dataset.peek = 'file does not exist'
             dataset.blurb = 'file purged from disc'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
             Checking for keyword - '<owl' in the first 200 lines.
         """
@@ -243,7 +245,7 @@ class Sbml(GenericXml):
     edam_data = "data_2024"
     edam_format = "format_2585"
 
-    def set_peek(self, dataset, is_multi_byte=False):
+    def set_peek(self, dataset):
         if not dataset.dataset.purged:
             dataset.peek = data.get_file_peek(dataset.file_name)
             dataset.blurb = "System Biology Markup Language SBML"
@@ -251,7 +253,7 @@ class Sbml(GenericXml):
             dataset.peek = 'file does not exist'
             dataset.blurb = 'file purged from disc'
 
-    def sniff_prefix(self, file_prefix):
+    def sniff_prefix(self, file_prefix: FilePrefix):
         """
             Checking for keyword - '<sbml' in the first 200 lines.
         """
