@@ -40,7 +40,7 @@ def upgrade(migrate_engine):
         return
 
     for (table, column) in BLOB_COLUMNS:
-        cmd = "ALTER TABLE {} MODIFY COLUMN {} MEDIUMBLOB;".format(table, column)
+        cmd = f"ALTER TABLE {table} MODIFY COLUMN {column} MEDIUMBLOB;"
         try:
             migrate_engine.execute(cmd)
         except Exception:
