@@ -12,7 +12,7 @@ log = logging.getLogger(__name__)
 
 def upgrade(migrate_engine):
     print(__doc__)
-    cmd = 'UPDATE history_dataset_association SET deleted=%s WHERE purged AND NOT deleted;' % engine_true(migrate_engine)
+    cmd = f'UPDATE history_dataset_association SET deleted={engine_true(migrate_engine)} WHERE purged AND NOT deleted;'
     try:
         migrate_engine.execute(cmd)
     except Exception:
