@@ -15,12 +15,13 @@ def upgrade(migrate_engine):
     print(__doc__)
     # drop first because sqlite does not support or_replace
     downgrade(migrate_engine)
-    create_view = CreateView(HistoryDatasetCollectionJobStateSummary)
+    view = HistoryDatasetCollectionJobStateSummary
+    create_view = CreateView(view.name, view.__view__)
     # print(str(create_view.compile(migrate_engine)))
     migrate_engine.execute(create_view)
 
 
 def downgrade(migrate_engine):
-    drop_view = DropView(HistoryDatasetCollectionJobStateSummary)
+    drop_view = DropView(HistoryDatasetCollectionJobStateSummary.name)
     # print(str(drop_view.compile(migrate_engine)))
     migrate_engine.execute(drop_view)
