@@ -11,7 +11,18 @@ import os
 from collections import (
     namedtuple,
 )
-from typing import Any, NamedTuple, Optional
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    NamedTuple,
+    Optional,
+    TYPE_CHECKING,
+    Union,
+)
+
+from sqlalchemy.orm.scoping import ScopedSession
 
 import galaxy.model
 from galaxy import util
@@ -20,24 +31,37 @@ from galaxy.exceptions import (
 )
 from galaxy.model.dataset_collections import builder
 from galaxy.model.tags import GalaxySessionlessTagHandler
+from galaxy.objectstore import ObjectStore
 from galaxy.util import (
     chunk_iterable,
     ExecutionTimer
 )
 from galaxy.util.hash_util import HASH_NAME_MAP
 
+if TYPE_CHECKING:
+    from galaxy.job_execution.output_collect import JobContext, SessionlessJobContext
+
+
 log = logging.getLogger(__name__)
 
 UNSET = object()
 DEFAULT_CHUNK_SIZE = 1000
 
 
+class MaxDiscoveredFilesExceededError(ValueError):
+    pass
+
+
 class ModelPersistenceContext(metaclass=abc.ABCMeta):
     """Class for creating datasets while finding files.
 
     This class implement the create_dataset method that takes care of populating metadata
     required for datasets and other potential model objects.
     """
+
+    max_discovered_files = float('inf')
+    discovered_file_count: int
+
     def create_dataset(
         self,
         ext,
@@ -122,6 +146,7 @@ class ModelPersistenceContext(metaclass=abc.ABCMeta):
         for source_dict in sources:
             source = galaxy.model.DatasetSource()
             source.source_uri = source_dict["source_uri"]
+            source.transform = source_dict.get("transform")
             primary_data.dataset.sources.append(source)
 
         for hash_dict in hashes:
@@ -134,7 +159,8 @@ class ModelPersistenceContext(metaclass=abc.ABCMeta):
             primary_data.created_from_basename = created_from_basename
 
         if tag_list:
-            self.tag_handler.add_tags_from_list(self.job.user, primary_data, tag_list, flush=False)
+            job = getattr(self, "job", None)
+            self.tag_handler.add_tags_from_list(job and job.user, primary_data, tag_list, flush=False)
 
         # If match specified a name use otherwise generate one from
         # designation.
@@ -230,7 +256,7 @@ class ModelPersistenceContext(metaclass=abc.ABCMeta):
             self._populate_elements(chunk=filenames.items(), name=name, root_collection_builder=root_collection_builder, metadata_source_name=metadata_source_name, final_job_state=final_job_state)
 
     def _populate_elements(self, chunk, name, root_collection_builder, metadata_source_name, final_job_state):
-        element_datasets = {'element_identifiers': [], 'datasets': [], 'tag_lists': [], 'paths': [], 'extra_files': []}
+        element_datasets: Dict[str, List[Any]] = {'element_identifiers': [], 'datasets': [], 'tag_lists': [], 'paths': [], 'extra_files': []}
         for filename, discovered_file in chunk:
             create_dataset_timer = ExecutionTimer()
             fields_match = discovered_file.match
@@ -308,7 +334,7 @@ class ModelPersistenceContext(metaclass=abc.ABCMeta):
     def add_tags_to_datasets(self, datasets, tag_lists):
         if any(tag_lists):
             for dataset, tags in zip(datasets, tag_lists):
-                self.tag_handler.add_tags_from_list(self.job.user, dataset, tags, flush=False)
+                self.tag_handler.add_tags_from_list(self.user, dataset, tags, flush=False)
 
     def update_object_store_with_datasets(self, datasets, paths, extra_files):
         for dataset, path, extra_file in zip(datasets, paths, extra_files):
@@ -330,6 +356,44 @@ class ModelPersistenceContext(metaclass=abc.ABCMeta):
         Return None otherwise.
         """
 
+    @abc.abstractproperty
+    def sa_session(self) -> Optional[ScopedSession]:
+        """If bound to a database, return the SQL Alchemy session.
+
+        Return None otherwise.
+        """
+
+    @abc.abstractproperty
+    def permission_provider(self) -> 'PermissionProvider':
+        """If bound to a database, return the SQL Alchemy session.
+
+        Return None otherwise.
+        """
+
+    def get_implicit_collection_jobs_association_id(self) -> Optional[str]:
+        """No-op, no job context."""
+        return None
+
+    @abc.abstractproperty
+    def job(self) -> Optional[galaxy.model.Job]:
+        """Return associated job object if bound to a job finish context connected to a database."""
+
+    @abc.abstractproperty
+    def metadata_source_provider(self) -> 'MetadataSourceProvider':
+        """Return associated MetadataSourceProvider object."""
+
+    @abc.abstractproperty
+    def object_store(self) -> ObjectStore:
+        """Return object store to use for populating discovered dataset contents."""
+
+    @abc.abstractproperty
+    def flush_per_n_datasets(self) -> Optional[int]:
+        pass
+
+    @property
+    def input_dbkey(self) -> str:
+        return '?'
+
     @abc.abstractmethod
     def add_library_dataset_to_folder(self, library_folder, ld):
         """Add library dataset to persisted library folder."""
@@ -357,6 +421,11 @@ class ModelPersistenceContext(metaclass=abc.ABCMeta):
     def flush(self):
         """If database bound, flush the persisted objects to ensure IDs."""
 
+    def increment_discovered_file_count(self):
+        self.discovered_file_count += 1
+        if self.discovered_file_count > self.max_discovered_files:
+            raise MaxDiscoveredFilesExceededError(f"Job generated more than maximum number ({self.max_discovered_files}) of output datasets")
+
 
 class PermissionProvider(metaclass=abc.ABCMeta):
     """Interface for working with permissions while importing datasets with ModelPersistenceContext."""
@@ -409,23 +478,47 @@ class SessionlessModelPersistenceContext(ModelPersistenceContext):
     """A variant of ModelPersistenceContext that persists to an export store instead of database directly."""
 
     def __init__(self, object_store, export_store, working_directory):
-        self.permission_provider = UnusedPermissionProvider()
-        self.metadata_source_provider = UnusedMetadataSourceProvider()
-        self.sa_session = None
-        self.object_store = object_store
+        self._permission_provider = UnusedPermissionProvider()
+        self._metadata_source_provider = UnusedMetadataSourceProvider()
+        self._object_store = object_store
         self.export_store = export_store
-        self.flush_per_n_datasets = None
-
+        self._flush_per_n_datasets = None
+        self.discovered_file_count = 0
+        self.max_discovered_files = float('inf')
         self.job_working_directory = working_directory  # TODO: rename...
 
     @property
     def tag_handler(self):
         return GalaxySessionlessTagHandler(self.sa_session)
 
+    @property
+    def sa_session(self):
+        return None
+
     @property
     def user(self):
         return None
 
+    @property
+    def job(self):
+        return None
+
+    @property
+    def permission_provider(self) -> UnusedPermissionProvider:
+        return self._permission_provider
+
+    @property
+    def metadata_source_provider(self) -> UnusedMetadataSourceProvider:
+        return self._metadata_source_provider
+
+    @property
+    def object_store(self) -> ObjectStore:
+        return self._object_store
+
+    @property
+    def flush_per_n_datasets(self) -> Optional[int]:
+        return self._flush_per_n_datasets
+
     def add_tags_to_datasets(self, datasets, tag_lists):
         user = galaxy.model.User()
         if any(tag_lists):
@@ -541,7 +634,7 @@ def persist_target_to_export_store(target_dict, export_store, object_store, work
         export_store.add_dataset_collection(hdca)
 
 
-def persist_elements_to_hdca(model_persistence_context, elements, hdca, collector=None):
+def persist_elements_to_hdca(model_persistence_context: Union['JobContext', 'SessionlessJobContext', SessionlessModelPersistenceContext], elements, hdca, collector=None):
     filenames = {}
 
     def add_to_discovered_files(elements, parent_identifiers=None):
@@ -550,7 +643,7 @@ def persist_elements_to_hdca(model_persistence_context, elements, hdca, collecto
             if "elements" in element:
                 add_to_discovered_files(element["elements"], parent_identifiers + [element["name"]])
             else:
-                discovered_file = discovered_file_for_element(element, model_persistence_context.job_working_directory, parent_identifiers, collector=collector)
+                discovered_file = discovered_file_for_element(element, model_persistence_context, parent_identifiers, collector=collector)
                 filenames[discovered_file.path] = discovered_file
 
     add_to_discovered_files(elements)
@@ -574,7 +667,7 @@ def persist_elements_to_folder(model_persistence_context, elements, library_fold
             nested_folder = model_persistence_context.create_library_folder(library_folder, name, description)
             persist_elements_to_folder(model_persistence_context, element["elements"], nested_folder)
         else:
-            discovered_file = discovered_file_for_element(element, model_persistence_context.job_working_directory)
+            discovered_file = discovered_file_for_element(element, model_persistence_context)
             fields_match = discovered_file.match
             designation = fields_match.designation
             visible = fields_match.visible
@@ -613,19 +706,20 @@ def persist_elements_to_folder(model_persistence_context, elements, library_fold
 def persist_hdas(elements, model_persistence_context, final_job_state='ok'):
     # discover files as individual datasets for the target history
     datasets = []
-    storage_callbacks = []
+    storage_callbacks: List[Callable] = []
 
     def collect_elements_for_history(elements):
         for element in elements:
             if "elements" in element:
                 collect_elements_for_history(element["elements"])
             else:
-                discovered_file = discovered_file_for_element(element, model_persistence_context.job_working_directory)
+                discovered_file = discovered_file_for_element(element, model_persistence_context)
                 fields_match = discovered_file.match
                 designation = fields_match.designation
                 ext = fields_match.ext
                 dbkey = fields_match.dbkey
                 info = element.get("info", None)
+                tag_list = element.get("tags")
                 link_data = discovered_file.match.link_data
 
                 # Create new primary dataset
@@ -654,6 +748,7 @@ def persist_hdas(elements, model_persistence_context, final_job_state='ok'):
                     filename=discovered_file.path,
                     extra_files=extra_files,
                     info=info,
+                    tag_list=tag_list,
                     link_data=link_data,
                     primary_data=primary_dataset,
                     sources=sources,
@@ -729,9 +824,10 @@ def replace_request_syntax_sugar(obj):
 DiscoveredFile = namedtuple('DiscoveredFile', ['path', 'collector', 'match'])
 
 
-def discovered_file_for_element(dataset, job_working_directory, parent_identifiers=None, collector=None):
+def discovered_file_for_element(dataset, model_persistence_context: Union['JobContext', 'SessionlessJobContext', SessionlessModelPersistenceContext], parent_identifiers=None, collector=None):
+    model_persistence_context.increment_discovered_file_count()
     parent_identifiers = parent_identifiers or []
-    target_directory = discover_target_directory(getattr(collector, "directory", None), job_working_directory)
+    target_directory = discover_target_directory(getattr(collector, "directory", None), model_persistence_context.job_working_directory)
     filename = dataset.get("filename")
     error_message = dataset.get("error_message")
     if error_message is None:
