@@ -1,12 +1,14 @@
 import logging
 import re
-from typing import Dict
+from typing import Dict, List, Optional, Tuple
 
-from sqlalchemy.orm.scoping import scoped_session
+from sqlalchemy.exc import IntegrityError
+from sqlalchemy.orm import sessionmaker
 from sqlalchemy.sql import select
 from sqlalchemy.sql.expression import func
 
 import galaxy.model
+from galaxy.model.scoped_session import galaxy_scoped_session
 from galaxy.util import (
     strip_control_characters,
     unicodify,
@@ -28,7 +30,7 @@ class TagHandler:
     Manages CRUD operations related to tagging objects.
     """
 
-    def __init__(self, sa_session: scoped_session) -> None:
+    def __init__(self, sa_session: galaxy_scoped_session) -> None:
         self.sa_session = sa_session
         # Minimum tag length.
         self.min_tag_len = 1
@@ -245,7 +247,20 @@ class TagHandler:
         return self.sa_session.query(galaxy.model.Tag).filter_by(name=tag_name).first()
 
     def _create_tag_instance(self, tag_name):
-        return galaxy.model.Tag(type=0, name=tag_name)
+        # For good performance caller should first check if there's already an appropriate tag
+        tag = galaxy.model.Tag(type=0, name=tag_name)
+        if not self.sa_session:
+            return tag
+        Session = sessionmaker(self.sa_session.bind)
+        with Session() as separate_session:
+            separate_session.add(tag)
+            try:
+                separate_session.commit()
+                separate_session.flush()
+            except IntegrityError:
+                # tag already exists, get from database
+                separate_session.rollback()
+        return self._get_tag(tag_name)
 
     def _get_or_create_tag(self, tag_str):
         """Get or create a Tag object from a tag string."""
@@ -290,7 +305,7 @@ class TagHandler:
         raw_tags = reg_exp.split(tag_str)
         return self.parse_tags_list(raw_tags)
 
-    def parse_tags_list(self, tags_list):
+    def parse_tags_list(self, tags_list: List[str]) -> List[Tuple[str, Optional[str]]]:
         """
         Return a list of tag tuples (name, value) pairs derived from a list.
         Method scrubs tag names and values as well.
@@ -341,13 +356,13 @@ class TagHandler:
             scrubbed_tag_list.append(self._scrub_tag_name(tag))
         return scrubbed_tag_list
 
-    def _get_name_value_pair(self, tag_str):
+    def _get_name_value_pair(self, tag_str) -> List[Optional[str]]:
         """Get name, value pair from a tag string."""
         # Use regular expression to parse name, value.
         if tag_str.startswith('#'):
             tag_str = f"name:{tag_str[1:]}"
         reg_exp = re.compile(f"[{self.key_value_separators}]")
-        name_value_pair = reg_exp.split(tag_str, 1)
+        name_value_pair: List[Optional[str]] = list(reg_exp.split(tag_str, 1))
         # Add empty slot if tag does not have value.
         if len(name_value_pair) < 2:
             name_value_pair.append(None)
@@ -355,7 +370,7 @@ class TagHandler:
 
 
 class GalaxyTagHandler(TagHandler):
-    def __init__(self, sa_session: scoped_session):
+    def __init__(self, sa_session: galaxy_scoped_session):
         from galaxy import model
         TagHandler.__init__(self, sa_session)
         self.item_tag_assoc_info["History"] = ItemTagAssocInfo(model.History,
@@ -411,7 +426,7 @@ class GalaxySessionlessTagHandler(GalaxyTagHandlerSession):
         return self.created_tags.get(tag_name)
 
     def get_tag_by_name(self, tag_name):
-        self.created_tags.get(tag_name)
+        return self.created_tags.get(tag_name)
 
 
 class CommunityTagHandler(TagHandler):
