@@ -1,6 +1,4 @@
-"""
-Galaxy Quotas
-"""
+"""Galaxy Quotas"""
 import logging
 
 import galaxy.util
@@ -8,19 +6,36 @@ import galaxy.util
 log = logging.getLogger(__name__)
 
 
-class NoQuotaAgent:
-    """Base quota agent, always returns no quota"""
+class QuotaAgent():  # metaclass=abc.ABCMeta
+    """Abstraction around querying Galaxy for quota available and used.
 
-    def __init__(self, model):
-        self.model = model
-        self.sa_session = model.context
+    Certain parts of the app that deal directly with modifying the quota assume more than
+    this interface - they assume the availability of the methods on DatabaseQuotaAgent that
+    implements this interface. But for read-only quota operations - such as checking available
+    quota or reporting it to users - methods defined on this interface should be sufficient
+    and the NoQuotaAgent should be a valid choice.
 
-    def get_quota(self, user, nice_size=False):
-        return None
+    Sticking to well annotated methods on this interface should make it clean and
+    possible to implement other backends for quota setting in the future such as managing
+    the quota in other apps (LDAP maybe?) or via configuration files.
+    """
 
-    @property
-    def default_quota(self):
-        return None
+    # TODO: make abstractmethod after they work better with mypy
+    def get_quota(self, user):
+        """Return quota in bytes or None if no quota is set."""
+
+    def get_quota_nice_size(self, user):
+        """Return quota as a human-readable string or 'unlimited' if no quota is set."""
+        quota_bytes = self.get_quota(user)
+        if quota_bytes is not None:
+            quota_str = galaxy.util.nice_size(quota_bytes)
+        else:
+            quota_str = 'unlimited'
+        return quota_str
+
+    # TODO: make abstractmethod after they work better with mypy
+    def get_percent(self, trans=None, user=False, history=False, usage=False, quota=False):
+        """Return the percentage of any storage quota applicable to the user/transaction."""
 
     def get_usage(self, trans=None, user=False, history=False):
         if trans:
@@ -34,17 +49,43 @@ class NoQuotaAgent:
             usage = user.total_disk_usage
         return usage
 
+    def is_over_quota(self, app, job, job_destination):
+        """Return True if the user or history is over quota for specified job.
+
+        job_destination unused currently but an important future application will
+        be admins and/or users dynamically specifying which object stores to use
+        and that will likely come in through the job destination.
+        """
+
+
+class NoQuotaAgent(QuotaAgent):
+    """Base quota agent, always returns no quota"""
+
+    def __init__(self):
+        pass
+
+    def get_quota(self, user):
+        return None
+
+    @property
+    def default_quota(self):
+        return None
+
     def get_percent(self, trans=None, user=False, history=False, usage=False, quota=False):
         return None
 
-    def get_user_quotas(self, user):
-        return []
+    def is_over_quota(self, app, job, job_destination):
+        return False
 
 
-class QuotaAgent(NoQuotaAgent):
+class DatabaseQuotaAgent(QuotaAgent):
     """Class that handles galaxy quotas"""
 
-    def get_quota(self, user, nice_size=False):
+    def __init__(self, model):
+        self.model = model
+        self.sa_session = model.context
+
+    def get_quota(self, user):
         """
         Calculated like so:
 
@@ -92,11 +133,6 @@ class QuotaAgent(NoQuotaAgent):
             rval = max + adjustment
             if rval <= 0:
                 rval = 0
-        if nice_size:
-            if rval is not None:
-                rval = galaxy.util.nice_size(rval)
-            else:
-                rval = 'unlimited'
         return rval
 
     @property
@@ -108,7 +144,7 @@ class QuotaAgent(NoQuotaAgent):
         return self._default_quota(self.model.DefaultQuotaAssociation.types.REGISTERED)
 
     def _default_quota(self, default_type):
-        dqa = self.sa_session.query(self.model.DefaultQuotaAssociation).filter(self.model.DefaultQuotaAssociation.table.c.type == default_type).first()
+        dqa = self.sa_session.query(self.model.DefaultQuotaAssociation).filter(self.model.DefaultQuotaAssociation.type == default_type).first()
         if not dqa:
             return None
         if dqa.quota.bytes < 0:
@@ -125,7 +161,7 @@ class QuotaAgent(NoQuotaAgent):
         for gqa in quota.groups:
             self.sa_session.delete(gqa)
         # Find the old default, assign the new quota if it exists
-        dqa = self.sa_session.query(self.model.DefaultQuotaAssociation).filter(self.model.DefaultQuotaAssociation.table.c.type == default_type).first()
+        dqa = self.sa_session.query(self.model.DefaultQuotaAssociation).filter(self.model.DefaultQuotaAssociation.type == default_type).first()
         if dqa:
             dqa.quota = quota
         # Or create if necessary
@@ -179,21 +215,25 @@ class QuotaAgent(NoQuotaAgent):
                 self.sa_session.add(gqa)
             self.sa_session.flush()
 
-    def get_user_quotas(self, user):
-        rval = []
-        if not user:
-            dqa = self.sa_session.query(self.model.DefaultQuotaAssociation) \
-                .filter(self.model.DefaultQuotaAssociation.table.c.type == self.model.DefaultQuotaAssociation.types.UNREGISTERED).first()
-            if dqa:
-                rval.append(dqa.quota)
-        else:
-            dqa = self.sa_session.query(self.model.DefaultQuotaAssociation) \
-                .filter(self.model.DefaultQuotaAssociation.table.c.type == self.model.DefaultQuotaAssociation.types.REGISTERED).first()
-            if dqa:
-                rval.append(dqa.quota)
-            for uqa in user.quotas:
-                rval.append(uqa.quota)
-            for group in [uga.group for uga in user.groups]:
-                for gqa in group.quotas:
-                    rval.append(gqa.quota)
-        return rval
+    def is_over_quota(self, app, job, job_destination):
+        quota = self.get_quota(job.user)
+        if quota is not None:
+            try:
+                usage = self.get_usage(user=job.user, history=job.history)
+                if usage > quota:
+                    return True
+            except AssertionError:
+                pass  # No history, should not happen with an anon user
+        return False
+
+
+def get_quota_agent(config, model) -> QuotaAgent:
+    quota_agent: QuotaAgent
+    if config.enable_quotas:
+        quota_agent = galaxy.quota.DatabaseQuotaAgent(model)
+    else:
+        quota_agent = galaxy.quota.NoQuotaAgent()
+    return quota_agent
+
+
+__all__ = ('get_quota_agent', 'NoQuotaAgent')
