@@ -1,5 +1,7 @@
 """This module contains general pydantic models and common schema field annotations for them."""
 
+import json
+import re
 from datetime import datetime
 from enum import Enum
 from typing import (
@@ -7,7 +9,7 @@ from typing import (
     Dict,
     List,
     Optional,
-    Tuple,
+    Set,
     Union,
 )
 
@@ -15,9 +17,9 @@ from pydantic import (
     AnyHttpUrl,
     AnyUrl,
     BaseModel,
+    ConstrainedStr,
     Extra,
     Field,
-    FilePath,
     Json,
     UUID4,
 )
@@ -29,9 +31,9 @@ from galaxy.model import (
     Job,
 )
 from galaxy.schema.fields import (
+    DecodedDatabaseIdField,
     EncodedDatabaseIdField,
     ModelClassField,
-    optional,
 )
 from galaxy.schema.types import RelativeUrl
 
@@ -85,20 +87,20 @@ DatasetStateField: Dataset.states = Field(
     description="The current state of this dataset.",
 )
 
-CreateTimeField: datetime = Field(
-    ...,
+CreateTimeField = Field(
     title="Create Time",
     description="The time and date this item was created.",
 )
 
-UpdateTimeField: datetime = Field(
-    ...,
+UpdateTimeField = Field(
     title="Update Time",
     description="The last time and date this item was updated.",
 )
 
-CollectionTypeField: str = Field(
-    ...,
+CollectionType = str  # str alias for now
+
+CollectionTypeField = Field(
+    default=None,
     title="Collection Type",
     description=(
         "The type of the collection, can be `list`, `paired`, or define subcollections using `:` "
@@ -133,12 +135,11 @@ ElementCountField: Optional[int] = Field(
 )
 
 PopulatedField: bool = Field(
-    ...,
     title="Populated",
     description="Whether the dataset collection elements (and any subcollections elements) were successfully populated.",
 )
 
-ElementsField: List['DCESummary'] = Field(
+ElementsField = Field(
     [],
     title="Elements",
     description="The summary information of each of the elements inside the dataset collection.",
@@ -162,6 +163,11 @@ GenomeBuildField: Optional[str] = Field(
     description="TODO",
 )
 
+ContentsUrlField = Field(
+    title="Contents URL",
+    description="The relative URL to access the contents of this History.",
+)
+
 
 class Model(BaseModel):
     """Base model definition with common configuration used by all derived models."""
@@ -241,9 +247,18 @@ class HistoryContentSource(str, Enum):
     new_collection = "new_collection"
 
 
+class DatasetCollectionInstanceType(str, Enum):
+    history = "history"
+    library = "library"
+
+
+class TagItem(ConstrainedStr):
+    regex = re.compile(r"^([^\s.:])+(.[^\s.:]+)*(:[^\s.:]+)?$")
+
+
 class TagCollection(Model):
     """Represents the collection of tags associated with an item."""
-    __root__: List[str] = Field(
+    __root__: List[TagItem] = Field(
         default=...,
         title="Tags",
         description="The collection of tags associated with an item.",
@@ -316,8 +331,7 @@ class Visualization(Model):  # TODO annotate this model
 class HistoryItemBase(Model):
     """Basic information provided by items contained in a History."""
     id: EncodedDatabaseIdField = EncodedEntityIdField
-    name: str = Field(
-        ...,
+    name: Optional[str] = Field(
         title="Name",
         description="The name of the item.",
     )
@@ -346,8 +360,11 @@ class HistoryItemBase(Model):
 
 class HistoryItemCommon(HistoryItemBase):
     """Common information provided by items contained in a History."""
-    type_id: str = Field(
-        ...,
+    class Config:
+        extra = Extra.allow
+
+    type_id: Optional[str] = Field(
+        default=None,
         title="Type - ID",
         description="The type and the encoded ID of this item. Used for caching.",
         example="dataset-616e371b2cc6c62e",
@@ -357,8 +374,8 @@ class HistoryItemCommon(HistoryItemBase):
         title="Type",
         description="The type of this item.",
     )
-    create_time: datetime = CreateTimeField
-    update_time: datetime = UpdateTimeField
+    create_time: Optional[datetime] = CreateTimeField
+    update_time: Optional[datetime] = UpdateTimeField
     url: RelativeUrl = RelativeUrlField
     tags: TagCollection
 
@@ -390,25 +407,28 @@ class HDAInaccessible(HistoryItemBase):
     state: Dataset.states = DatasetStateField
 
 
+HdaLddaField = Field(
+    DatasetSourceType.hda,
+    const=True,
+    title="HDA or LDDA",
+    description="Whether this dataset belongs to a history (HDA) or a library (LDDA).",
+    deprecated=False  # TODO Should this field be deprecated in favor of model_class?
+)
+
+
 class HDADetailed(HDASummary):
     """History Dataset Association detailed information."""
     model_class: str = ModelClassField(HDA_MODEL_CLASS_NAME)
-    hda_ldda: DatasetSourceType = Field(
-        DatasetSourceType.hda,
-        const=True,
-        title="HDA or LDDA",
-        description="Whether this dataset belongs to a history (HDA) or a library (LDDA).",
-        deprecated=False  # TODO Should this field be deprecated in favor of model_class?
-    )
+    hda_ldda: DatasetSourceType = HdaLddaField
     accessible: bool = AccessibleField
     genome_build: Optional[str] = GenomeBuildField
-    misc_info: str = Field(
-        ...,
+    misc_info: Optional[str] = Field(
+        default=None,
         title="Miscellaneous Information",
         description="TODO",
     )
-    misc_blurb: str = Field(
-        ...,
+    misc_blurb: Optional[str] = Field(
+        default=None,
         title="Miscellaneous Blurb",
         description="TODO",
     )
@@ -427,13 +447,23 @@ class HDADetailed(HDASummary):
         title="Resubmitted",
         description="Whether the job creating this dataset has been resubmitted.",
     )
-    metadata: Any = Field(  # TODO: create pydantic model for metadata?
-        ...,
+    metadata: Optional[Any] = Field(  # TODO: create pydantic model for metadata?
+        default=None,
         title="Metadata",
         description="The metadata associated with this dataset.",
     )
+    metadata_dbkey: Optional[str] = Field(
+        "?",
+        title="Metadata DBKey",
+        description="TODO",
+    )
+    metadata_data_lines: int = Field(
+        0,
+        title="Metadata Data Lines",
+        description="TODO",
+    )
     meta_files: List[MetadataFile] = Field(
-        [],
+        ...,
         title="Metadata Files",
         description="Collection of metadata files associated with this dataset.",
     )
@@ -443,8 +473,8 @@ class HDADetailed(HDASummary):
         description="The fully qualified name of the class implementing the data type of this dataset.",
         example="galaxy.datatypes.data.Text"
     )
-    peek: str = Field(
-        ...,
+    peek: Optional[str] = Field(
+        default=None,
         title="Peek",
         description="A few lines of contents from the start of the file.",
     )
@@ -464,24 +494,24 @@ class HDADetailed(HDASummary):
         title="Permissions",
         description="Role-based access and manage control permissions for the dataset.",
     )
-    file_name: FilePath = Field(
-        ...,
+    file_name: Optional[str] = Field(
+        default=None,
         title="File Name",
         description="The full path to the dataset file.",
     )
     display_apps: List[DisplayApp] = Field(
-        [],
+        ...,
         title="Display Applications",
         description="Contains new-style display app urls.",
     )
     display_types: List[DisplayApp] = Field(
-        [],
+        ...,
         title="Legacy Display Applications",
         description="Contains old-style display app urls.",
         deprecated=False,  # TODO: Should this field be deprecated in favor of display_apps?
     )
     visualizations: List[Visualization] = Field(
-        [],
+        ...,
         title="Visualizations",
         description="The collection of visualizations that can be applied to this dataset.",
     )
@@ -542,24 +572,41 @@ class HDABeta(HDADetailed):  # TODO: change HDABeta name to a more appropriate o
     pass
 
 
-@optional
-class UpdateHDAPayload(HDABeta):
-    """Used for updating a particular HDA. All fields are optional."""
-    pass
-
-
 class DCSummary(Model):
     """Dataset Collection summary information."""
     model_class: str = ModelClassField(DC_MODEL_CLASS_NAME)
     id: EncodedDatabaseIdField = EncodedEntityIdField
     create_time: datetime = CreateTimeField
     update_time: datetime = UpdateTimeField
-    collection_type: str = CollectionTypeField
+    collection_type: CollectionType = CollectionTypeField
     populated_state: DatasetCollection.populated_states = PopulatedStateField
     populated_state_message: Optional[str] = PopulatedStateMessageField
     element_count: Optional[int] = ElementCountField
 
 
+class HDAObject(Model):
+    """History Dataset Association Object"""
+    id: EncodedDatabaseIdField = EncodedEntityIdField
+    model_class: str = ModelClassField(HDA_MODEL_CLASS_NAME)
+    state: Dataset.states = DatasetStateField
+    hda_ldda: DatasetSourceType = HdaLddaField
+    history_id: EncodedDatabaseIdField = HistoryIdField
+
+    class Config:
+        extra = Extra.allow  # Can contain more fields like metadata_*
+
+
+class DCObject(Model):
+    """Dataset Collection Object"""
+    id: EncodedDatabaseIdField = EncodedEntityIdField
+    model_class: str = ModelClassField(DC_MODEL_CLASS_NAME)
+    collection_type: CollectionType = CollectionTypeField
+    populated: Optional[bool] = PopulatedField
+    element_count: Optional[int] = ElementCountField
+    contents_url: Optional[RelativeUrl] = ContentsUrlField
+    elements: List['DCESummary'] = ElementsField
+
+
 class DCESummary(Model):
     """Dataset Collection Element summary information."""
     id: EncodedDatabaseIdField = EncodedEntityIdField
@@ -579,13 +626,16 @@ class DCESummary(Model):
         title="Element Type",
         description="The type of the element. Used to interpret the `object` field.",
     )
-    object: Union[HDASummary, DCSummary] = Field(
+    object: Union[HDAObject, HDADetailed, DCObject] = Field(
         ...,
         title="Object",
         description="The element's specific data depending on the value of `element_type`.",
     )
 
 
+DCObject.update_forward_refs()
+
+
 class DCDetailed(DCSummary):
     """Dataset Collection detailed information."""
     populated: bool = PopulatedField
@@ -601,7 +651,7 @@ class HDCASummary(HistoryItemCommon):
         title="Type",
         description="This is always `collection` for dataset collections.",
     )
-    collection_type: str = CollectionTypeField
+    collection_type: CollectionType = CollectionTypeField
     populated_state: DatasetCollection.populated_states = PopulatedStateField
     populated_state_message: Optional[str] = PopulatedStateMessageField
     element_count: Optional[int] = ElementCountField
@@ -615,11 +665,7 @@ class HDCASummary(HistoryItemCommon):
         title="Job Source Type",
         description="The type of job (model class) that produced this dataset collection. Used to track the state of the job.",
     )
-    contents_url: RelativeUrl = Field(
-        ...,
-        title="Contents URL",
-        description="The relative URL to access the contents of this dataset collection.",
-    )
+    contents_url: RelativeUrl = ContentsUrlField
 
 
 class HDCADetailed(HDCASummary):
@@ -628,40 +674,6 @@ class HDCADetailed(HDCASummary):
     elements: List[DCESummary] = ElementsField
 
 
-@optional
-class UpdateHDCAPayload(HDCADetailed):
-    """Used for updating a particular HDCA. All fields are optional."""
-    pass
-
-
-class UpdateHistoryContentsBatchPayload(BaseModel):
-    class Config:
-        use_enum_values = True  # when using .dict()
-        allow_population_by_field_name = True
-        extra = Extra.allow  # Allow any additional field
-
-    items: List[Union[UpdateHDAPayload, UpdateHDCAPayload]] = Field(
-        ...,
-        title="Items",
-        description="A list of content items to update with the changes.",
-    )
-    deleted: Optional[bool] = Field(
-        default=False,
-        title="Deleted",
-        description=(
-            "This will check the uploading state if not deleting (i.e: deleted=False), "
-            "otherwise cannot delete uploading files, so it will raise an error."
-        ),
-    )
-    visible: Optional[bool] = Field(
-        default=False,
-        title="Visible",
-        description=(
-            "Show or hide history contents"
-        ),
-    )
-
-
 class HistoryBase(BaseModel):
     """Provides basic configuration for all the History models."""
     class Config:
@@ -669,6 +681,50 @@ class HistoryBase(BaseModel):
         extra = Extra.allow  # Allow any other extra fields
 
 
+class UpdateContentItem(HistoryBase):
+    """Used for updating a particular HDA. All fields are optional."""
+    history_content_type: HistoryContentType = Field(
+        ...,
+        title="Content Type",
+        description="The type of this item.",
+    )
+    id: EncodedDatabaseIdField = EncodedEntityIdField
+
+
+class UpdateHistoryContentsBatchPayload(HistoryBase):
+    """Contains property values that will be updated for all the history `items` provided."""
+
+    items: List[UpdateContentItem] = Field(
+        ...,
+        title="Items",
+        description="A list of content items to update with the changes.",
+    )
+
+    class Config:
+        schema_extra = {
+            "example": {
+                "items": [
+                    {
+                        "history_content_type": "dataset",
+                        "id": "string"
+                    }
+                ],
+                "visible": False,
+            }
+        }
+
+
+class UpdateHistoryContentsPayload(HistoryBase):
+    """Contains arbitrary property values that will be updated for a particular history item."""
+    class Config:
+        schema_extra = {
+            "example": {
+                "visible": False,
+                "annotation": "Test",
+            }
+        }
+
+
 class HistorySummary(HistoryBase):
     """History summary information."""
     model_class: str = ModelClassField(HISTORY_MODEL_CLASS_NAME)
@@ -723,11 +779,7 @@ HistoryStateIds = Dict[Dataset.states, List[EncodedDatabaseIdField]]
 
 class HistoryDetailed(HistorySummary):  # Equivalent to 'dev-detailed' view, which seems the default
     """History detailed information."""
-    contents_url: RelativeUrl = Field(
-        ...,
-        title="Contents URL",
-        description="The relative URL to access the contents of this History.",
-    )
+    contents_url: RelativeUrl = ContentsUrlField
     size: int = Field(
         ...,
         title="Size",
@@ -811,6 +863,14 @@ class HistoryBeta(HistoryDetailed):
     )
 
 
+AnyHistoryView = Union[
+    HistoryBeta, HistoryDetailed, HistorySummary,
+    # Any will cover those cases in which only specific `keys` are requested
+    # otherwise the validation will fail because the required fields are not returned
+    Any,
+]
+
+
 class ExportHistoryArchivePayload(Model):
     gzip: Optional[bool] = Field(
         default=True,
@@ -848,6 +908,43 @@ class ExportHistoryArchivePayload(Model):
     )
 
 
+class SortByEnum(str, Enum):
+    create_time = "create_time"
+    update_time = "update_time"
+    none = None
+
+
+class InvocationIndexPayload(Model):
+    workflow_id: Optional[DecodedDatabaseIdField] = Field(title="Workflow ID", description="Return only invocations for this Workflow ID")
+    history_id: Optional[DecodedDatabaseIdField] = Field(title="History ID", description="Return only invocations for this History ID")
+    job_id: Optional[DecodedDatabaseIdField] = Field(title="Job ID", description="Return only invocations for this Job ID")
+    user_id: Optional[DecodedDatabaseIdField] = Field(title="User ID", description="Return invocations for this User ID")
+    sort_by: Optional[SortByEnum] = Field(
+        title="Sort By",
+        description="Sort Workflow Invocations by this attribute"
+    )
+    sort_desc: bool = Field(
+        default=False,
+        descritpion="Sort in descending order?"
+    )
+    include_terminal: bool = Field(
+        default=True,
+        description="Set to false to only include terminal Invocations."
+    )
+    limit: Optional[int] = Field(
+        default=100,
+        lt=1000,
+    )
+    offset: Optional[int] = Field(
+        default=0,
+        description="Number of invocations to skip"
+    )
+    instance: bool = Field(
+        default=False,
+        description="Is provided workflow id for Workflow instead of StoredWorkflow?"
+    )
+
+
 class CreateHistoryPayload(Model):
     name: Optional[str] = Field(
         default=None,
@@ -875,8 +972,6 @@ class CreateHistoryPayload(Model):
         title="Archive Source",
         description=(
             "The URL that will generate the archive to import when `archive_type='url'`. "
-            # This seems a bit odd but the create history action expects `archive_source` to be != None
-            "When importing from a file using `archive_file`, please set `archive_source=''`."
         ),
     )
     archive_type: Optional[HistoryImportArchiveSourceType] = Field(
@@ -887,11 +982,82 @@ class CreateHistoryPayload(Model):
     archive_file: Optional[Any] = Field(
         default=None,
         title="Archive File",
-        description="Detailed file information when importing the history from a file.",
+        description="Uploaded file information when importing the history from a file.",
     )
 
 
-class JobExportHistoryArchive(Model):
+class CollectionElementIdentifier(Model):
+    name: Optional[str] = Field(
+        None,
+        title="Name",
+        description="The name of the element.",
+    )
+    src: ColletionSourceType = Field(
+        ...,
+        title="Source",
+        description="The source of the element.",
+    )
+    id: Optional[EncodedDatabaseIdField] = Field(
+        default=None,
+        title="ID",
+        description="The encoded ID of the element.",
+    )
+    collection_type: Optional[CollectionType] = CollectionTypeField
+    element_identifiers: Optional[List['CollectionElementIdentifier']] = Field(
+        default=None,
+        title="Element Identifiers",
+        description="List of elements that should be in the new sub-collection.",
+    )
+    tags: Optional[List[str]] = Field(
+        default=None,
+        title="Tags",
+        description="The list of tags associated with the element.",
+    )
+
+
+# Required for self-referencing models
+# See https://pydantic-docs.helpmanual.io/usage/postponed_annotations/#self-referencing-models
+CollectionElementIdentifier.update_forward_refs()
+
+
+class CreateNewCollectionPayload(Model):
+    collection_type: Optional[CollectionType] = CollectionTypeField
+    element_identifiers: Optional[List[CollectionElementIdentifier]] = Field(
+        default=None,
+        title="Element Identifiers",
+        description="List of elements that should be in the new collection.",
+    )
+    name: Optional[str] = Field(
+        default=None,
+        title="Name",
+        description="The name of the new collection.",
+    )
+    hide_source_items: Optional[bool] = Field(
+        default=False,
+        title="Hide Source Items",
+        description="Whether to mark the original HDAs as hidden.",
+    )
+    copy_elements: Optional[bool] = Field(
+        default=False,
+        title="Copy Elements",
+        description="Whether to create a copy of the source HDAs for the new collection.",
+    )
+    instance_type: Optional[DatasetCollectionInstanceType] = Field(
+        default=DatasetCollectionInstanceType.history,
+        title="Instance Type",
+        description="The type of the instance, either `history` (default) or `library`.",
+    )
+    history_id: Optional[EncodedDatabaseIdField] = Field(
+        default=None,
+        description="The ID of the history that will contain the collection. Required if `instance_type=history`.",
+    )
+    folder_id: Optional[EncodedDatabaseIdField] = Field(
+        default=None,
+        description="The ID of the history that will contain the collection. Required if `instance_type=library`.",
+    )
+
+
+class JobExportHistoryArchiveModel(Model):
     id: EncodedDatabaseIdField = Field(
         ...,
         title="ID",
@@ -934,6 +1100,10 @@ class JobExportHistoryArchive(Model):
     )
 
 
+class JobExportHistoryArchiveCollection(Model):
+    __root__: List[JobExportHistoryArchiveModel]
+
+
 class LabelValuePair(BaseModel):
     """Generic Label/Value pair model."""
     label: str = Field(
@@ -1058,6 +1228,10 @@ class HDCABeta(HDCADetailed):  # TODO: change HDCABeta name to a more appropriat
         title="Job State Summary",
         description="Overview of the job states working inside the dataset collection.",
     )
+    elements_datatypes: Set[str] = Field(
+        ...,
+        description="A set containing all the different element datatypes in the collection."
+    )
 
 
 class JobBaseModel(Model):
@@ -1928,7 +2102,8 @@ class RoleListModel(BaseModel):
 
 # The tuple should probably be another proper model instead?
 # Keeping it as a Tuple for now for backward compatibility
-RoleNameIdTuple = Tuple[str, EncodedDatabaseIdField]
+# TODO: Use Tuple again when https://github.com/tiangolo/fastapi/issues/3665 is fixed upstream
+RoleNameIdTuple = List[str]  # Tuple[str, EncodedDatabaseIdField]
 
 # Group_Roles -----------------------------------------------------------------
 
@@ -2319,7 +2494,7 @@ class DatasetAssociationRoles(Model):
 
 class UpdateDatasetPermissionsPayload(Model):
     action: Optional[DatasetPermissionAction] = Field(
-        ...,
+        DatasetPermissionAction.set_permissions,
         title="Action",
         description="Indicates what action should be performed on the dataset.",
     )
@@ -2343,14 +2518,381 @@ class UpdateDatasetPermissionsPayload(Model):
     )
 
 
-class DeleteHDCAResult(Model):
+class CustomHistoryItem(Model):
+    """Can contain any serializable property of the item.
+
+    Allows arbitrary custom keys to be specified in the serialization
+    parameters without a particular view (predefined set of keys).
+    """
+    class Config:
+        extra = Extra.allow
+
+
+AnyHDA = Union[HDABeta, HDADetailed, HDASummary]
+AnyHDCA = Union[HDCABeta, HDCADetailed, HDCASummary]
+AnyHistoryContentItem = Union[
+    AnyHDA,
+    AnyHDCA,
+    CustomHistoryItem,
+]
+
+
+AnyJobStateSummary = Union[
+    JobStateSummary,
+    ImplicitCollectionJobsStateSummary,
+    WorkflowInvocationStateSummary,
+]
+
+HistoryArchiveExportResult = Union[JobExportHistoryArchiveModel, JobIdResponse]
+
+
+class DeleteHistoryContentPayload(BaseModel):
+    purge: bool = Field(
+        default=False,
+        title="Purge",
+        description="Whether to remove the dataset from storage. Datasets will only be removed from storage once all HDAs or LDDAs that refer to this datasets are deleted.",
+    )
+    recursive: bool = Field(
+        default=False,
+        title="Recursive",
+        description="When deleting a dataset collection, whether to also delete containing datasets.",
+    )
+
+
+class DeleteHistoryContentResult(CustomHistoryItem):
+    """Contains minimum information about the deletion state of a history item.
+
+    Can also contain any other properties of the item."""
     id: EncodedDatabaseIdField = Field(
         ...,
         title="ID",
-        description="The encoded ID of the collection.",
+        description="The encoded ID of the history item.",
     )
     deleted: bool = Field(
         ...,
         title="Deleted",
-        description="True if the collection was successfully deleted.",
+        description="True if the item was successfully deleted.",
+    )
+    purged: Optional[bool] = Field(
+        default=None,
+        title="Purged",
+        description="True if the item was successfully removed from disk.",
+    )
+
+
+class HistoryContentsArchiveDryRunResult(BaseModel):
+    """
+    Contains a collection of filepath/filename entries that represent
+    the contents that would have been included in the archive.
+    This is returned when the `dry_run` flag is active when
+    creating an archive with the contents of the history.
+
+    This is used for debugging purposes.
+    """
+    # TODO: Use Tuple again when https://github.com/tiangolo/fastapi/issues/3665 is fixed upstream
+    __root__: List[List[str]]  # List[Tuple[str, str]]
+
+
+class ContentsNearStats(BaseModel):
+    """Stats used by the `contents_near` endpoint."""
+    matches: int
+    matches_up: int
+    matches_down: int
+    total_matches: int
+    total_matches_up: int
+    total_matches_down: int
+    max_hid: Optional[int] = None
+    min_hid: Optional[int] = None
+    history_size: int
+    history_empty: bool
+
+    def to_headers(self) -> Dict[str, str]:
+        """Converts all field values to json strings.
+
+        The headers values need to be json strings or updating the response
+        headers will raise encoding errors."""
+        headers = {}
+        for key, val in self:
+            headers[key] = json.dumps(val)
+        return headers
+
+
+class HistoryContentsResult(Model):
+    """Collection of history content items.
+    Can contain different views and kinds of items.
+    """
+    __root__: List[AnyHistoryContentItem]
+
+
+class ContentsNearResult(BaseModel):
+    contents: HistoryContentsResult
+    stats: ContentsNearStats
+
+
+# Sharing -----------------------------------------------------------------
+
+
+class SharingOptions(str, Enum):
+    """Options for sharing resources that may have restricted access to all or part of their contents."""
+    make_public = "make_public"
+    make_accessible_to_shared = "make_accessible_to_shared"
+    no_changes = "no_changes"
+
+
+class ShareWithExtra(BaseModel):
+    can_share: bool = Field(
+        False,
+        title="Can Share",
+        description="Indicates whether the resource can be directly shared or requires further actions.",
+    )
+
+    class Config:
+        extra = Extra.allow
+
+
+UserIdentifier = Union[EncodedDatabaseIdField, str]
+
+
+class ShareWithPayload(BaseModel):
+    user_ids: List[UserIdentifier] = Field(
+        ...,
+        title="User Identifiers",
+        description=(
+            "A collection of encoded IDs (or email addresses) of users "
+            "that this resource will be shared with."
+        ),
+    )
+    share_option: Optional[SharingOptions] = Field(
+        None,
+        title="Share Option",
+        description=(
+            "User choice for sharing resources which its contents may be restricted:\n"
+            " - None: The user did not choose anything yet or no option is needed.\n"
+            f" - {SharingOptions.make_public}: The contents of the resource will be made publicly accessible.\n"
+            f" - {SharingOptions.make_accessible_to_shared}: This will automatically create a new `sharing role` allowing protected contents to be accessed only by the desired users.\n"
+            f" - {SharingOptions.no_changes}: This won't change the current permissions for the contents. The user which this resource will be shared may not be able to access all its contents.\n"
+        ),
+    )
+
+
+class SetSlugPayload(BaseModel):
+    new_slug: str = Field(
+        ...,
+        title="New Slug",
+        description="The slug that will be used to access this shared item.",
+    )
+
+
+class UserEmail(BaseModel):
+    id: EncodedDatabaseIdField = Field(
+        ...,
+        title="User ID",
+        description="The encoded ID of the user.",
+    )
+    email: str = Field(
+        ...,
+        title="Email",
+        description="The email of the user.",
+    )
+
+
+class SharingStatus(BaseModel):
+    id: EncodedDatabaseIdField = Field(
+        ...,
+        title="ID",
+        description="The encoded ID of the resource to be shared.",
+    )
+    title: str = Field(
+        ...,
+        title="Title",
+        description="The title or name of the resource.",
+    )
+    importable: bool = Field(
+        ...,
+        title="Importable",
+        description="Whether this resource can be published using a link.",
+    )
+    published: bool = Field(
+        ...,
+        title="Published",
+        description="Whether this resource is currently published.",
+    )
+    users_shared_with: List[UserEmail] = Field(
+        [],
+        title="Users shared with",
+        description="The list of encoded ids for users the resource has been shared.",
+    )
+    username_and_slug: Optional[str] = Field(
+        None,
+        title="Username and slug",
+        description="The relative URL in the form of /u/{username}/{resource_single_char}/{slug}",
+    )
+
+
+class ShareWithStatus(SharingStatus):
+    errors: List[str] = Field(
+        [],
+        title="Errors",
+        description="Collection of messages indicating that the resource was not shared with some (or all users) due to an error.",
+    )
+    extra: Optional[ShareWithExtra] = Field(
+        None,
+        title="Extra",
+        description=(
+            "Optional extra information about this shareable resource that may be of interest. "
+            "The contents of this field depend on the particular resource."
+        ),
+    )
+
+
+class HDABasicInfo(BaseModel):
+    id: EncodedDatabaseIdField
+    name: str
+
+
+class ShareHistoryExtra(ShareWithExtra):
+    can_change: List[HDABasicInfo] = Field(
+        [],
+        title="Can Change",
+        description=(
+            "A collection of datasets that are not accessible by one or more of the target users "
+            "and that can be made accessible for others by the user sharing the history."
+        ),
+    )
+    cannot_change: List[HDABasicInfo] = Field(
+        [],
+        title="Cannot Change",
+        description=(
+            "A collection of datasets that are not accessible by one or more of the target users "
+            "and that cannot be made accessible for others by the user sharing the history."
+        ),
+    )
+    accessible_count: int = Field(
+        0,
+        title="Accessible Count",
+        description=(
+            "The number of datasets in the history that are public or accessible by all the target users."
+        ),
+    )
+
+# Pages -------------------------------------------------------
+
+
+class PageContentFormat(str, Enum):
+    markdown = "markdown"
+    html = "html"
+
+
+ContentFormatField: PageContentFormat = Field(
+    default=PageContentFormat.html,
+    title="Content format",
+    description="Either `markdown` or `html`.",
+)
+
+ContentField: Optional[str] = Field(
+    default="",
+    title="Content",
+    description="Raw text contents of the first page revision (type dependent on content_format).",
+)
+
+
+class PageSummaryBase(BaseModel):
+    title: str = Field(
+        ...,  # Required
+        title="Title",
+        description="The name of the page",
+    )
+    slug: str = Field(
+        ...,  # Required
+        title="Identifier",
+        description="The title slug for the page URL, must be unique.",
+        regex=r"^[a-z0-9\-]+$",
+    )
+
+
+class CreatePagePayload(PageSummaryBase):
+    content_format: PageContentFormat = ContentFormatField
+    content: Optional[str] = ContentField
+    annotation: Optional[str] = Field(
+        default=None,
+        title="Annotation",
+        description="Annotation that will be attached to the page.",
+    )
+    invocation_id: Optional[EncodedDatabaseIdField] = Field(
+        None,
+        title="Workflow invocation ID",
+        description="Encoded ID used by workflow generated reports.",
+    )
+
+    class Config:
+        use_enum_values = True  # When using .dict()
+        extra = Extra.allow  # Allow any other extra fields
+
+
+class PageSummary(PageSummaryBase):
+    id: EncodedDatabaseIdField = Field(
+        ...,  # Required
+        title="ID",
+        description="Encoded ID of the Page.",
+    )
+    model_class: str = Field(
+        ...,  # Required
+        title="Model class",
+        description="The class of the model associated with the ID.",
+        example="Page",
+    )
+    username: str = Field(
+        ...,  # Required
+        title="Username",
+        description="The name of the user owning this Page.",
+    )
+    published: bool = Field(
+        ...,  # Required
+        title="Published",
+        description="Whether this Page has been published.",
+    )
+    importable: bool = Field(
+        ...,  # Required
+        title="Importable",
+        description="Whether this Page can be imported.",
+    )
+    deleted: bool = Field(
+        ...,  # Required
+        title="Deleted",
+        description="Whether this Page has been deleted.",
+    )
+    latest_revision_id: EncodedDatabaseIdField = Field(
+        ...,  # Required
+        title="Latest revision ID",
+        description="The encoded ID of the last revision of this Page.",
+    )
+    revision_ids: List[EncodedDatabaseIdField] = Field(
+        ...,  # Required
+        title="List of revisions",
+        description="The history with the encoded ID of each revision of the Page.",
+    )
+
+
+class PageDetails(PageSummary):
+    content_format: PageContentFormat = ContentFormatField
+    content: Optional[str] = ContentField
+    generate_version: Optional[str] = Field(
+        None,
+        title="Galaxy Version",
+        description="The version of Galaxy this page was generated with.",
+    )
+    generate_time: Optional[str] = Field(
+        None,
+        title="Generate Date",
+        description="The date this page was generated.",
+    )
+
+    class Config:
+        extra = Extra.allow  # Allow any other extra fields
+
+
+class PageSummaryList(BaseModel):
+    __root__: List[PageSummary] = Field(
+        default=[],
+        title='List with summary information of Pages.',
     )
