@@ -46,7 +46,7 @@ def hash_password_PBKDF2(password):
     hashed_password = pbkdf2_bin(password, salt, COST_FACTOR, KEY_LENGTH, HASH_FUNCTION)
     encoded_password = unicodify(b64encode(hashed_password))
     # Format
-    return 'PBKDF2${}${}${}${}'.format(HASH_FUNCTION, COST_FACTOR, unicodify(salt), encoded_password)
+    return f'PBKDF2${HASH_FUNCTION}${COST_FACTOR}${unicodify(salt)}${encoded_password}'
 
 
 def check_password_PBKDF2(guess, hashed):
