@@ -4,13 +4,15 @@ import random
 import unittest
 import uuid
 from tempfile import NamedTemporaryFile
+from typing import List
 
 import pytest
-from sqlalchemy import inspect
+from sqlalchemy import inspect, select
 
 import galaxy.datatypes.registry
 import galaxy.model
 import galaxy.model.mapping as mapping
+from galaxy import model
 from galaxy.model.database_utils import create_database
 from galaxy.model.metadata import MetadataTempFile
 from galaxy.model.security import GalaxyRBACAgent
@@ -29,6 +31,7 @@ skip_if_not_postgres_base = pytest.mark.skipif(
 
 
 class BaseModelTestCase(unittest.TestCase):
+    model: mapping.GalaxyModelMapping
 
     @classmethod
     def _db_uri(cls):
@@ -69,8 +72,6 @@ class BaseModelTestCase(unittest.TestCase):
 class MappingTests(BaseModelTestCase):
 
     def test_annotations(self):
-        model = self.model
-
         u = model.User(email="annotator@example.com", password="password")
         self.persist(u)
 
@@ -104,7 +105,7 @@ class MappingTests(BaseModelTestCase):
         self.persist(h)
         persist_and_check_annotation(model.HistoryAnnotationAssociation, history=h)
 
-        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=model.session)
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
         self.persist(d1)
         persist_and_check_annotation(model.HistoryDatasetAssociationAnnotationAssociation, hda=d1)
 
@@ -128,8 +129,6 @@ class MappingTests(BaseModelTestCase):
         persist_and_check_annotation(model.LibraryDatasetCollectionAnnotationAssociation, library_dataset_collection=library_dataset_collection)
 
     def test_ratings(self):
-        model = self.model
-
         user_email = "rater@example.com"
         u = model.User(email=user_email, password="password")
         self.persist(u)
@@ -152,7 +151,7 @@ class MappingTests(BaseModelTestCase):
         self.persist(h)
         persist_and_check_rating(model.HistoryRatingAssociation, h)
 
-        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=model.session)
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
         self.persist(d1)
         persist_and_check_rating(model.HistoryDatasetAssociationRatingAssociation, d1)
 
@@ -181,22 +180,22 @@ class MappingTests(BaseModelTestCase):
             assert isinstance(item.get_display_name(), str)
             assert item.get_display_name() == name
 
-        ldda = self.model.LibraryDatasetDatasetAssociation(name='ldda_name')
+        ldda = model.LibraryDatasetDatasetAssociation(name='ldda_name')
         assert_display_name_converts_to_unicode(ldda, 'ldda_name')
 
-        hda = self.model.HistoryDatasetAssociation(name='hda_name')
+        hda = model.HistoryDatasetAssociation(name='hda_name')
         assert_display_name_converts_to_unicode(hda, 'hda_name')
 
-        history = self.model.History(name='history_name')
+        history = model.History(name='history_name')
         assert_display_name_converts_to_unicode(history, 'history_name')
 
-        library = self.model.Library(name='library_name')
+        library = model.Library(name='library_name')
         assert_display_name_converts_to_unicode(library, 'library_name')
 
-        library_folder = self.model.LibraryFolder(name='library_folder')
+        library_folder = model.LibraryFolder(name='library_folder')
         assert_display_name_converts_to_unicode(library_folder, 'library_folder')
 
-        history = self.model.History(
+        history = model.History(
             name='Hello₩◎ґʟⅾ'
         )
 
@@ -205,11 +204,11 @@ class MappingTests(BaseModelTestCase):
         assert history.get_display_name() == 'Hello₩◎ґʟⅾ'
 
     def test_hda_to_library_dataset_dataset_association(self):
-        u = self.model.User(email="mary@example.com", password="password")
-        hda = self.model.HistoryDatasetAssociation(name='hda_name')
+        u = model.User(email="mary@example.com", password="password")
+        hda = model.HistoryDatasetAssociation(name='hda_name')
         self.persist(hda)
         trans = collections.namedtuple('trans', 'user')
-        target_folder = self.model.LibraryFolder(name='library_folder')
+        target_folder = model.LibraryFolder(name='library_folder')
         ldda = hda.to_library_dataset_dataset_association(
             trans=trans(user=u),
             target_folder=target_folder,
@@ -235,103 +234,97 @@ class MappingTests(BaseModelTestCase):
         assert target_folder.item_count == 1
 
     def test_tags(self):
-        model = self.model
-
-        my_tag = model.Tag(name="Test Tag")
+        TAG_NAME = 'Test Tag'
+        my_tag = model.Tag(name=TAG_NAME)
         u = model.User(email="tagger@example.com", password="password")
         self.persist(my_tag, u)
 
-        def tag_and_test(taggable_object, tag_association_class, backref_name):
-            assert len(getattr(self.query(model.Tag).filter(model.Tag.name == "Test Tag").all()[0], backref_name)) == 0
+        def tag_and_test(taggable_object, tag_association_class):
+            q = select(tag_association_class).join(model.Tag).where(model.Tag.name == TAG_NAME)
+
+            assert len(self.model.session.execute(q).all()) == 0
 
             tag_association = tag_association_class()
             tag_association.tag = my_tag
             taggable_object.tags = [tag_association]
             self.persist(tag_association, taggable_object)
 
-            assert len(getattr(self.query(model.Tag).filter(model.Tag.name == "Test Tag").all()[0], backref_name)) == 1
+            assert len(self.model.session.execute(q).all()) == 1
 
-        sw = model.StoredWorkflow()
-        sw.user = u
-        tag_and_test(sw, model.StoredWorkflowTagAssociation, "tagged_stored_workflows")
+        sw = model.StoredWorkflow(user=u)
+        tag_and_test(sw, model.StoredWorkflowTagAssociation)
 
         h = model.History(name="History for Tagging", user=u)
-        tag_and_test(h, model.HistoryTagAssociation, "tagged_histories")
+        tag_and_test(h, model.HistoryTagAssociation)
 
-        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=model.session)
-        tag_and_test(d1, model.HistoryDatasetAssociationTagAssociation, "tagged_history_dataset_associations")
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
+        tag_and_test(d1, model.HistoryDatasetAssociationTagAssociation)
 
-        page = model.Page()
-        page.user = u
-        tag_and_test(page, model.PageTagAssociation, "tagged_pages")
+        page = model.Page(user=u)
+        tag_and_test(page, model.PageTagAssociation)
 
-        visualization = model.Visualization()
-        visualization.user = u
-        tag_and_test(visualization, model.VisualizationTagAssociation, "tagged_visualizations")
+        visualization = model.Visualization(user=u)
+        tag_and_test(visualization, model.VisualizationTagAssociation)
 
         dataset_collection = model.DatasetCollection(collection_type="paired")
         history_dataset_collection = model.HistoryDatasetCollectionAssociation(collection=dataset_collection)
-        tag_and_test(history_dataset_collection, model.HistoryDatasetCollectionTagAssociation, "tagged_history_dataset_collections")
+        tag_and_test(history_dataset_collection, model.HistoryDatasetCollectionTagAssociation)
 
         library_dataset_collection = model.LibraryDatasetCollectionAssociation(collection=dataset_collection)
-        tag_and_test(library_dataset_collection, model.LibraryDatasetCollectionTagAssociation, "tagged_library_dataset_collections")
+        tag_and_test(library_dataset_collection, model.LibraryDatasetCollectionTagAssociation)
 
     def test_collection_get_interface(self):
-        model = self.model
         u = model.User(email="mary@example.com", password="password")
         h1 = model.History(name="History 1", user=u)
-        d1 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=model.session)
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=self.model.session)
         c1 = model.DatasetCollection(collection_type="list")
         elements = 100
         dces = [model.DatasetCollectionElement(collection=c1, element=d1, element_identifier=f"{i}", element_index=i) for i in range(elements)]
         self.persist(u, h1, d1, c1, *dces, flush=False, expunge=False)
-        model.session.flush()
+        self.model.session.flush()
         for i in range(elements):
             assert c1[i] == dces[i]
 
     def test_dataset_instance_order(self):
-        model = self.model
         u = model.User(email="mary@example.com", password="password")
         h1 = model.History(name="History 1", user=u)
         elements = []
         list_pair = model.DatasetCollection(collection_type="list:paired")
         for i in range(20):
             pair = model.DatasetCollection(collection_type="pair")
-            forward = model.HistoryDatasetAssociation(extension="txt", history=h1, name=f"forward_{i}", create_dataset=True, sa_session=model.session)
-            reverse = model.HistoryDatasetAssociation(extension="bam", history=h1, name=f"reverse_{i}", create_dataset=True, sa_session=model.session)
+            forward = model.HistoryDatasetAssociation(extension="txt", history=h1, name=f"forward_{i}", create_dataset=True, sa_session=self.model.session)
+            reverse = model.HistoryDatasetAssociation(extension="bam", history=h1, name=f"reverse_{i}", create_dataset=True, sa_session=self.model.session)
             dce1 = model.DatasetCollectionElement(collection=pair, element=forward, element_identifier=f"forward_{i}", element_index=1)
             dce2 = model.DatasetCollectionElement(collection=pair, element=reverse, element_identifier=f"reverse_{i}", element_index=2)
             to_persist = [(forward, reverse), (dce1, dce2)]
             self.persist(pair)
-            for item in to_persist:
+            for pair_item in to_persist:
                 if i % 2:
-                    self.persist(item[0])
-                    self.persist(item[1])
+                    self.persist(pair_item[0])
+                    self.persist(pair_item[1])
                 else:
-                    self.persist(item[1])
-                    self.persist(item[0])
+                    self.persist(pair_item[1])
+                    self.persist(pair_item[0])
             elements.append(model.DatasetCollectionElement(collection=list_pair, element=pair, element_index=i, element_identifier=str(i)))
         self.persist(list_pair)
         random.shuffle(elements)
         for item in elements:
             self.persist(item)
-        forward = []
-        reverse = []
+        forward_hdas: List[model.HistoryDatasetAssociation] = []
+        reverse_hdas: List[model.HistoryDatasetAssociation] = []
         for i, dataset_instance in enumerate(list_pair.dataset_instances):
             if i % 2:
-                reverse.append(dataset_instance)
+                reverse_hdas.append(dataset_instance)
             else:
-                forward.append(dataset_instance)
-        assert all(d.name == f"forward_{i}" for i, d in enumerate(forward))
-        assert all(d.name == f"reverse_{i}" for i, d in enumerate(reverse))
+                forward_hdas.append(dataset_instance)
+        assert all(d.name == f"forward_{i}" for i, d in enumerate(forward_hdas))
+        assert all(d.name == f"reverse_{i}" for i, d in enumerate(reverse_hdas))
 
     def test_collections_in_histories(self):
-        model = self.model
-
         u = model.User(email="mary@example.com", password="password")
         h1 = model.History(name="History 1", user=u)
-        d1 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=model.session)
-        d2 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=model.session)
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=self.model.session)
+        d2 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=self.model.session)
 
         c1 = model.DatasetCollection(collection_type="pair")
         hc1 = model.HistoryDatasetCollectionAssociation(history=h1, collection=c1, name="HistoryCollectionTest1")
@@ -348,8 +341,6 @@ class MappingTests(BaseModelTestCase):
         assert loaded_dataset_collection["right"] == dce2
 
     def test_collections_in_library_folders(self):
-        model = self.model
-
         u = model.User(email="mary2@example.com", password="password")
         lf = model.LibraryFolder(name="RootFolder")
         library = model.Library(name="Library1", root_folder=lf)
@@ -370,10 +361,9 @@ class MappingTests(BaseModelTestCase):
         # assert loaded_dataset_collection.collection_type == "pair"
 
     def test_nested_collection_attributes(self):
-        model = self.model
         u = model.User(email="mary2@example.com", password="password")
         h1 = model.History(name="History 1", user=u)
-        d1 = model.HistoryDatasetAssociation(extension="bam", history=h1, create_dataset=True, sa_session=model.session)
+        d1 = model.HistoryDatasetAssociation(extension="bam", history=h1, create_dataset=True, sa_session=self.model.session)
         index = NamedTemporaryFile("w")
         index.write("cool bam index")
         index2 = NamedTemporaryFile("w")
@@ -384,7 +374,7 @@ class MappingTests(BaseModelTestCase):
         assert d1.metadata.bam_csi_index
         assert isinstance(d1.metadata.bam_index, model.MetadataFile)
         assert isinstance(d1.metadata.bam_csi_index, model.MetadataFile)
-        d2 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=model.session)
+        d2 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=self.model.session)
         c1 = model.DatasetCollection(collection_type='paired')
         dce1 = model.DatasetCollectionElement(collection=c1, element=d1, element_identifier="forward", element_index=0)
         dce2 = model.DatasetCollectionElement(collection=c1, element=d2, element_identifier="reverse", element_index=1)
@@ -393,12 +383,12 @@ class MappingTests(BaseModelTestCase):
         c3 = model.DatasetCollection(collection_type="list:list")
         c4 = model.DatasetCollection(collection_type="list:list:paired")
         dce4 = model.DatasetCollectionElement(collection=c4, element=c2, element_identifier="outer_list", element_index=0)
-        model.session.add_all([d1, d2, c1, dce1, dce2, c2, dce3, c3, c4, dce4])
-        model.session.flush()
+        self.model.session.add_all([d1, d2, c1, dce1, dce2, c2, dce3, c3, c4, dce4])
+        self.model.session.flush()
         q = c2._get_nested_collection_attributes(element_attributes=('element_identifier',), hda_attributes=('extension',), dataset_attributes=('state',))
         assert [(r.keys()) for r in q] == [['element_identifier_0', 'element_identifier_1', 'extension', 'state'], ['element_identifier_0', 'element_identifier_1', 'extension', 'state']]
         assert q.all() == [('inner_list', 'forward', 'bam', 'new'), ('inner_list', 'reverse', 'txt', 'new')]
-        q = c2._get_nested_collection_attributes(return_entities=(model.HistoryDatasetAssociation,))
+        q = c2._get_nested_collection_attributes(return_entities=(model.HistoryDatasetAssociation, ))
         assert q.all() == [d1, d2]
         q = c2._get_nested_collection_attributes(return_entities=(model.HistoryDatasetAssociation, model.Dataset))
         assert q.all() == [(d1, d1.dataset), (d2, d2.dataset)]
@@ -418,74 +408,66 @@ class MappingTests(BaseModelTestCase):
         assert c4.element_identifiers_extensions_and_paths == [(('outer_list', 'inner_list', 'forward'), 'bam', 'mock_dataset_14.dat'), (('outer_list', 'inner_list', 'reverse'), 'txt', 'mock_dataset_14.dat')]
 
     def test_dataset_dbkeys_and_extensions_summary(self):
-        model = self.model
         u = model.User(email="mary2@example.com", password="password")
         h1 = model.History(name="History 1", user=u)
-        d1 = model.HistoryDatasetAssociation(extension="bam", dbkey="hg19", history=h1, create_dataset=True, sa_session=model.session)
-        d2 = model.HistoryDatasetAssociation(extension="txt", dbkey="hg19", history=h1, create_dataset=True, sa_session=model.session)
+        d1 = model.HistoryDatasetAssociation(extension="bam", dbkey="hg19", history=h1, create_dataset=True, sa_session=self.model.session)
+        d2 = model.HistoryDatasetAssociation(extension="txt", dbkey="hg19", history=h1, create_dataset=True, sa_session=self.model.session)
         c1 = model.DatasetCollection(collection_type='paired')
         dce1 = model.DatasetCollectionElement(collection=c1, element=d1, element_identifier="forward", element_index=0)
         dce2 = model.DatasetCollectionElement(collection=c1, element=d2, element_identifier="reverse", element_index=1)
         hdca = model.HistoryDatasetCollectionAssociation(collection=c1, history=h1)
-        model.session.add_all([d1, d2, c1, dce1, dce2, hdca])
-        model.session.flush()
+        self.model.session.add_all([d1, d2, c1, dce1, dce2, hdca])
+        self.model.session.flush()
         assert hdca.dataset_dbkeys_and_extensions_summary[0] == {"hg19"}
         assert hdca.dataset_dbkeys_and_extensions_summary[1] == {"bam", "txt"}
 
     def test_populated_optimized_ok(self):
-        model = self.model
         u = model.User(email="mary2@example.com", password="password")
         h1 = model.History(name="History 1", user=u)
-        d1 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=model.session)
-        d2 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=model.session)
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=self.model.session)
+        d2 = model.HistoryDatasetAssociation(extension="txt", history=h1, create_dataset=True, sa_session=self.model.session)
         c1 = model.DatasetCollection(collection_type='paired')
         dce1 = model.DatasetCollectionElement(collection=c1, element=d1, element_identifier="forward", element_index=0)
         dce2 = model.DatasetCollectionElement(collection=c1, element=d2, element_identifier="reverse", element_index=1)
-        model.session.add_all([d1, d2, c1, dce1, dce2])
-        model.session.flush()
+        self.model.session.add_all([d1, d2, c1, dce1, dce2])
+        self.model.session.flush()
         assert c1.populated
         assert c1.populated_optimized
 
     def test_populated_optimized_empty_list_list_ok(self):
-        model = self.model
         c1 = model.DatasetCollection(collection_type='list')
         c2 = model.DatasetCollection(collection_type='list:list')
         dce1 = model.DatasetCollectionElement(collection=c2, element=c1, element_identifier="empty_list", element_index=0)
-        model.session.add_all([c1, c2, dce1])
-        model.session.flush()
+        self.model.session.add_all([c1, c2, dce1])
+        self.model.session.flush()
         assert c1.populated
         assert c1.populated_optimized
         assert c2.populated
         assert c2.populated_optimized
 
     def test_populated_optimized_list_list_not_populated(self):
-        model = self.model
         c1 = model.DatasetCollection(collection_type='list')
         c1.populated_state = False
         c2 = model.DatasetCollection(collection_type='list:list')
         dce1 = model.DatasetCollectionElement(collection=c2, element=c1, element_identifier="empty_list", element_index=0)
-        model.session.add_all([c1, c2, dce1])
-        model.session.flush()
+        self.model.session.add_all([c1, c2, dce1])
+        self.model.session.flush()
         assert not c1.populated
         assert not c1.populated_optimized
         assert not c2.populated
         assert not c2.populated_optimized
 
     def test_default_disk_usage(self):
-        model = self.model
-
         u = model.User(email="disk_default@test.com", password="password")
         self.persist(u)
         u.adjust_total_disk_usage(1)
         u_id = u.id
         self.expunge()
-        user_reload = model.session.query(model.User).get(u_id)
+        user_reload = self.model.session.query(model.User).get(u_id)
         assert user_reload.disk_usage == 1
 
     def test_basic(self):
-        model = self.model
-
-        original_user_count = len(model.session.query(model.User).all())
+        original_user_count = len(self.model.session.query(model.User).all())
 
         # Make some changes and commit them
         u = model.User(email="james@foo.bar.baz", password="password")
@@ -497,20 +479,20 @@ class MappingTests(BaseModelTestCase):
         self.persist(u, h1, h2)
         # q1 = model.Query( "h2->q1" )
         metadata = dict(chromCol=1, startCol=2, endCol=3)
-        d1 = model.HistoryDatasetAssociation(extension="interval", metadata=metadata, history=h2, create_dataset=True, sa_session=model.session)
+        d1 = model.HistoryDatasetAssociation(extension="interval", metadata=metadata, history=h2, create_dataset=True, sa_session=self.model.session)
         # h2.queries.append( q1 )
         # h2.queries.append( model.Query( "h2->q2" ) )
         self.persist(d1)
 
         # Check
-        users = model.session.query(model.User).all()
+        users = self.model.session.query(model.User).all()
         assert len(users) == original_user_count + 1
         user = [user for user in users if user.email == "james@foo.bar.baz"][0]
         assert user.email == "james@foo.bar.baz"
         assert user.password == "password"
         assert len(user.histories) == 1
         assert user.histories[0].name == "History 1"
-        hists = model.session.query(model.History).all()
+        hists = self.model.session.query(model.History).all()
         hist0 = [history for history in hists if history.name == "History 1"][0]
         hist1 = [history for history in hists if history.name == "H" * 255][0]
         assert hist0.name == "History 1"
@@ -524,7 +506,7 @@ class MappingTests(BaseModelTestCase):
         # Do an update and check
         hist1.name = "History 2b"
         self.expunge()
-        hists = model.session.query(model.History).all()
+        hists = self.model.session.query(model.History).all()
         hist0 = [history for history in hists if history.name == "History 1"][0]
         hist1 = [history for history in hists if history.name == "History 2b"][0]
         assert hist0.name == "History 1"
@@ -533,22 +515,20 @@ class MappingTests(BaseModelTestCase):
 
     def test_metadata_spec(self):
         metadata = dict(chromCol=1, startCol=2, endCol=3)
-        d = self.model.HistoryDatasetAssociation(extension="interval", metadata=metadata, sa_session=self.model.session)
+        d = model.HistoryDatasetAssociation(extension="interval", metadata=metadata, sa_session=self.model.session)
         assert d.metadata.chromCol == 1
         assert d.metadata.anyAttribute is None
         assert 'items' not in d.metadata
 
     def test_dataset_job_relationship(self):
-        model = self.model
         dataset = model.Dataset()
         job = model.Job()
         dataset.job = job
         self.persist(job, dataset)
-        loaded_dataset = model.session.query(model.Dataset).filter(model.Dataset.id == dataset.id).one()
+        loaded_dataset = self.model.session.query(model.Dataset).filter(model.Dataset.id == dataset.id).one()
         assert loaded_dataset.job_id == job.id
 
     def test_jobs(self):
-        model = self.model
         u = model.User(email="jobtest@foo.bar.baz", password="password")
         job = model.Job()
         job.user = u
@@ -556,11 +536,10 @@ class MappingTests(BaseModelTestCase):
 
         self.persist(u, job)
 
-        loaded_job = model.session.query(model.Job).filter(model.Job.user == u).first()
+        loaded_job = self.model.session.query(model.Job).filter(model.Job.user == u).first()
         assert loaded_job.tool_id == "cat1"
 
     def test_job_metrics(self):
-        model = self.model
         u = model.User(email="jobtest@foo.bar.baz", password="password")
         job = model.Job()
         job.user = u
@@ -582,18 +561,16 @@ class MappingTests(BaseModelTestCase):
         assert len(task.text_metrics[1].metric_value) <= 1023
 
     def test_tasks(self):
-        model = self.model
         u = model.User(email="jobtest@foo.bar.baz", password="password")
         job = model.Job()
         task = model.Task(job=job, working_directory="/tmp", prepare_files_cmd="split.sh")
         job.user = u
         self.persist(u, job, task)
 
-        loaded_task = model.session.query(model.Task).filter(model.Task.job == job).first()
+        loaded_task = self.model.session.query(model.Task).filter(model.Task.job == job).first()
         assert loaded_task.prepare_input_files_cmd == "split.sh"
 
     def test_history_contents(self):
-        model = self.model
         u = model.User(email="contents@foo.bar.baz", password="password")
         # gs = model.GalaxySession()
         h1 = model.History(name="HistoryContentsHistory1", user=u)
@@ -608,7 +585,7 @@ class MappingTests(BaseModelTestCase):
         self.session().flush()
 
         def contents_iter_names(**kwds):
-            history = model.context.query(model.History).filter(
+            history = self.model.context.query(model.History).filter(
                 model.History.name == "HistoryContentsHistory1"
             ).first()
             return list(map(lambda hda: hda.name, history.contents_iter(**kwds)))
@@ -625,7 +602,6 @@ class MappingTests(BaseModelTestCase):
         assert contents_iter_names(ids=[d1.id, d3.id]) == ["1", "3"]
 
     def test_history_audit(self):
-        model = self.model
         u = model.User(email="contents@foo.bar.baz", password="password")
         h1 = model.History(name="HistoryAuditHistory", user=u)
         h2 = model.History(name="HistoryAuditHistory", user=u)
@@ -645,7 +621,7 @@ class MappingTests(BaseModelTestCase):
         self.new_hda(h1, name="1")
         self.new_hda(h2, name="2")
         self.session().flush()
-        # db_next_hid modifies history, plus trigger on HDA means 2 additional audit rows per history
+        # _next_hid modifies history, plus trigger on HDA means 2 additional audit rows per history
 
         h1_audits = get_audit_table_entries(h1)
         h2_audits = get_audit_table_entries(h2)
@@ -665,7 +641,6 @@ class MappingTests(BaseModelTestCase):
         assert h2_audits[0] == h2_latest
 
     def _non_empty_flush(self):
-        model = self.model
         lf = model.LibraryFolder(name="RootFolder")
         session = self.session()
         session.add(lf)
@@ -746,7 +721,6 @@ class MappingTests(BaseModelTestCase):
         assert 'id' not in inspect(galaxy_model_object_new).unloaded
 
     def test_workflows(self):
-        model = self.model
         user = model.User(
             email="testworkflows@bx.psu.edu",
             password="password"
@@ -848,13 +822,13 @@ class MappingTests(BaseModelTestCase):
         security_agent = GalaxyRBACAgent(self.model)
 
         def check_private_role(private_role, email):
-            assert private_role.type == self.model.Role.types.PRIVATE
+            assert private_role.type == model.Role.types.PRIVATE
             assert len(private_role.users) == 1
             assert private_role.name == email
             assert private_role.description == "Private Role for " + email
 
         email = "rule_user_1@example.com"
-        u = self.model.User(email=email, password="password")
+        u = model.User(email=email, password="password")
         self.persist(u)
 
         role = security_agent.get_private_user_role(u)
@@ -864,7 +838,7 @@ class MappingTests(BaseModelTestCase):
         check_private_role(role, email)
 
         email = "rule_user_2@example.com"
-        u = self.model.User(email=email, password="password")
+        u = model.User(email=email, password="password")
         self.persist(u)
         role = security_agent.get_private_user_role(u)
         assert role is None
@@ -882,8 +856,8 @@ class MappingTests(BaseModelTestCase):
 
         u_from, u_to, u_other = self._three_users("private_share_role")
 
-        h = self.model.History(name="History for Annotation", user=u_from)
-        d1 = self.model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
+        h = model.History(name="History for Annotation", user=u_from)
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
         self.persist(h, d1)
 
         security_agent.privately_share_dataset(d1.dataset, [u_to])
@@ -894,8 +868,8 @@ class MappingTests(BaseModelTestCase):
         security_agent = GalaxyRBACAgent(self.model)
         u_from, u_to, u_other = self._three_users("make_dataset_public")
 
-        h = self.model.History(name="History for Annotation", user=u_from)
-        d1 = self.model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
+        h = model.History(name="History for Annotation", user=u_from)
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
         self.persist(h, d1)
 
         security_agent.privately_share_dataset(d1.dataset, [u_to])
@@ -908,8 +882,8 @@ class MappingTests(BaseModelTestCase):
         security_agent = GalaxyRBACAgent(self.model)
         u_from, _, u_other = self._three_users("set_all_perms")
 
-        h = self.model.History(name="History for Annotation", user=u_from)
-        d1 = self.model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
+        h = model.History(name="History for Annotation", user=u_from)
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
         self.persist(h, d1)
 
         role = security_agent.get_private_user_role(u_from, auto_create=True)
@@ -925,8 +899,8 @@ class MappingTests(BaseModelTestCase):
         security_agent = GalaxyRBACAgent(self.model)
         u_from, u_to, u_other = self._three_users("can_manage_dataset")
 
-        h = self.model.History(name="History for Prevent Sharing", user=u_from)
-        d1 = self.model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
+        h = model.History(name="History for Prevent Sharing", user=u_from)
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
         self.persist(h, d1)
 
         self._make_owned(security_agent, u_from, d1)
@@ -938,22 +912,47 @@ class MappingTests(BaseModelTestCase):
         security_agent = GalaxyRBACAgent(self.model)
         u_from, _, u_other = self._three_users("can_manage_dataset_ps")
 
-        h = self.model.History(name="History for Prevent Sharing", user=u_from)
-        d1 = self.model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
+        h = model.History(name="History for Prevent Sharing", user=u_from)
+        d1 = model.HistoryDatasetAssociation(extension="txt", history=h, create_dataset=True, sa_session=self.model.session)
         self.persist(h, d1)
 
         self._make_private(security_agent, u_from, d1)
         assert security_agent.can_manage_dataset(u_from.all_roles(), d1.dataset)
         assert not security_agent.can_manage_dataset(u_other.all_roles(), d1.dataset)
 
+    def test_history_hid_counter_is_expired_after_next_hid_call(self):
+        u = model.User(email="hid_abuser@example.com", password="password")
+        h = model.History(name="History for hid testing", user=u)
+        self.persist(u, h)
+        state = inspect(h)
+        assert h.hid_counter == 1
+        assert 'hid_counter' not in state.unloaded
+        assert 'id' not in state.unloaded
+
+        h._next_hid()
+
+        assert 'hid_counter' in state.unloaded  # this attribute has been expired
+        assert 'id' not in state.unloaded  # but other attributes have NOT been expired
+        assert h.hid_counter == 2  # check this last: this causes thie hid_counter to be reloaded
+
+    def test_next_hid(self):
+        u = model.User(email="hid_abuser@example.com", password="password")
+        h = model.History(name="History for hid testing", user=u)
+        self.persist(u, h)
+        assert h.hid_counter == 1
+        h._next_hid()
+        assert h.hid_counter == 2
+        h._next_hid(n=3)
+        assert h.hid_counter == 5
+
     def _three_users(self, suffix):
         email_from = f"user_{suffix}e1@example.com"
         email_to = f"user_{suffix}e2@example.com"
         email_other = f"user_{suffix}e3@example.com"
 
-        u_from = self.model.User(email=email_from, password="password")
-        u_to = self.model.User(email=email_to, password="password")
-        u_other = self.model.User(email=email_other, password="password")
+        u_from = model.User(email=email_from, password="password")
+        u_to = model.User(email=email_to, password="password")
+        u_other = model.User(email=email_other, password="password")
         self.persist(u_from, u_to, u_other)
         return u_from, u_to, u_other
 
@@ -971,7 +970,7 @@ class MappingTests(BaseModelTestCase):
         security_agent.set_all_dataset_permissions(hda.dataset, permissions)
 
     def new_hda(self, history, **kwds):
-        return history.add_dataset(self.model.HistoryDatasetAssociation(create_dataset=True, sa_session=self.model.session, **kwds))
+        return history.add_dataset(model.HistoryDatasetAssociation(create_dataset=True, sa_session=self.model.session, **kwds))
 
 
 @skip_if_not_postgres_base
@@ -981,6 +980,7 @@ class PostgresMappingTests(MappingTests):
     def _db_uri(cls):
         base = os.environ.get("GALAXY_TEST_UNIT_MAPPING_URI_POSTGRES_BASE")
         dbname = "gxtest" + str(uuid.uuid4())
+        assert base
         postgres_url = base + dbname
         create_database(postgres_url)
         return postgres_url
